@@ -285,20 +285,6 @@ function clone_template(template) {
   return node;
 }
 
-function set_tvar(template, tvar, value) {
-  var node;
-  node = find_child(template, tvar);
-  if (node === null) {
-    throw new Error("Template does not contain variable " + tvar);
-  }
-  node.innerHTML = "";
-  if (typeof value !== "object") {
-    node.appendChild(document.createTextNode(value));
-  } else {
-    node.appendChild(value);
-  }
-}
-
 function make_hit_popup(container, block, match, seg, hit) {
   return function(evt) {
     "use strict";
