@@ -76,36 +76,6 @@ function post_load_setup() {
   delayed_process_draw_tasks();
 }
 
-//function make_help_button(container, help_topic) {
-//  container.appendChild(help_button(help_topic));
-//}
-
-function make_description(container, description) {
-  "use strict";
-  var header, box, paragraphs, p, lines, i, j;
-  container.innerHTML = "";
-  if (description) {
-    header = document.createElement("h2");
-    header.className = "mainh pad2";
-    header.id = "description";
-    header.appendChild(document.createTextNode("Description"));
-    container.appendChild(header);
-    box = document.createElement("div");
-    box.className = "box";
-    container.appendChild(box);
-    paragraphs = data['description'].split("\n\n");
-    for (i = 0; i < paragraphs.length; i++) {
-      p = document.createElement("p");
-      lines = paragraphs[i].split("\n");
-      for (j = 0; j < lines.length; j++) {
-        if (j != 0) p.appendChild(document.createElement('br'));
-        p.appendChild(document.createTextNode(lines[j]));
-      }
-      box.appendChild(p);
-    }
-  }
-}
-
 function make_clustered_motifs(container) {
   var groups = data['groups'];
   var motifs = data['motifs'];
@@ -214,20 +184,17 @@ function make_alignment_row(alphabet, seq_len, motifs, tbody, align_len, align,
   if (!more) {// make unaligned logos
     unaligned = document.createElement("span");
     unaligned.className = "unaligned";
-    unaligned.appendChild(make_logo(alphabet, pspm, false, 
-          0, "normal_logo"));
+    unaligned.appendChild(make_logo(alphabet, pspm, -90, false, 0, "normal_logo"));
     if (alphabet.has_complement()) {
-      unaligned.appendChild(make_logo(alphabet, pspm, true, 
-          0, "flipped_logo"));
+      unaligned.appendChild(make_logo(alphabet, pspm, -90, true, 0, "flipped_logo"));
     }
     cell.appendChild(unaligned);
   }
   aligned = document.createElement("span");
   aligned.className = "aligned";
-  aligned.appendChild(make_logo(alphabet, pspm, align["rc"], 
-        align["offset"], "normal_logo"));
+  aligned.appendChild(make_logo(alphabet, pspm, -90, align["rc"], align["offset"], "normal_logo"));
   if (alphabet.has_complement()) {
-    aligned.appendChild(make_logo(alphabet, pspm, !align["rc"], 
+    aligned.appendChild(make_logo(alphabet, pspm, -90, !align["rc"], 
         align_len - (align["offset"] + motif["len"]), "flipped_logo"));
   }
   cell.appendChild(aligned);
@@ -483,29 +450,6 @@ function make_motif_db_listing(container) {
   container.appendChild(tbl);
 }
 
-var DelayLogoTask = function(logo, canvas) {
-  this.logo = logo;
-  this.canvas = canvas;
-};
-
-DelayLogoTask.prototype.run = function () {
-  draw_logo_on_canvas(this.logo, this.canvas, false);
-};
-
-
-function make_logo(alphabet, pspm, rc, offset, className) {
-  if (rc) pspm = pspm.copy().reverse_complement(alphabet);
-  var logo = new Logo(alphabet, "");
-  logo.add_pspm(pspm, offset);
-  var canvas = document.createElement('canvas');
-  canvas.height = 90;
-  canvas.width = 0;
-  canvas.className = className;
-  size_logo_on_canvas(logo, canvas, false);
-  add_draw_task(canvas, new DelayLogoTask(logo, canvas));
-  return canvas;
-}
-
 var DelayCentrimoPlotTask = function(graph, canvas) {
   this.graph = graph;
   this.canvas = canvas;
@@ -800,7 +744,6 @@ function logos_end_transition(e) {
   }
 }
 
-
 /*
  * sync_table_columns
  *
@@ -833,4 +776,3 @@ function sync_table_columns(tables) {
     }
   }
 }
-
