@@ -1,7 +1,7 @@
+var current_program = "MEME";
+var current_alphabet = new Alphabet(data.alphabet, data.background.freqs);
 var current_motif = 0;
-var meme_alphabet = new Alphabet(data.alphabet, data.background.freqs);
-var new_icon_src = "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABwAAAAQCAMAAAAyEe/dAAAACVBMVEX//wAAAAD////s2cV/AAAAdUlEQVQYlXVRBxLAIAhL+P+jC2HZhXcBZEWEldDsZcLIcAhHWWnK8SDcWQhMFUHdAQ1CqQ5+CWPmlHojl+nCJNRtzu4qRc3IUzmTVpXYK0nox0z0PI1stgchdK7lEv7ekhvalw8WW547Gyzedt/2/gLx8WXjXF/1AYFriNAWAAAAAElFTkSuQmCC";
-
+//var new_icon_src = "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABwAAAAQCAMAAAAyEe/dAAAACVBMVEX//wAAAAD////s2cV/AAAAdUlEQVQYlXVRBxLAIAhL+P+jC2HZhXcBZEWEldDsZcLIcAhHWWnK8SDcWQhMFUHdAQ1CqQ5+CWPmlHojl+nCJNRtzu4qRc3IUzmTVpXYK0nox0z0PI1stgchdK7lEv7ekhvalw8WW547Gyzedt/2/gLx8WXjXF/1AYFriNAWAAAAAElFTkSuQmCC";
 
 var DelayLogoTask = function(logo, canvas) {
   this.logo = logo;
@@ -12,71 +12,6 @@ DelayLogoTask.prototype.run = function () {
   draw_logo_on_canvas(this.logo, this.canvas, false);
 };
 
-function motif_pspm(index) {
-  var motif, pwm, psm, name, ltrim, rtrim, nsites, evalue;
-  // get motif
-  motif = data["motifs"][index];
-  // get motif paramters
-  pwm = motif["pwm"]; 
-  psm = motif["psm"];
-  name = "" + (index + 1); ltrim = 0; rtrim = 0; 
-  nsites = motif["nsites"]; evalue = motif["evalue"];
-  // make pspm
-  return new Pspm(pwm, name, ltrim, rtrim, nsites, evalue, psm);
-}
-
-function motif_count_matrix(index) {
-  return motif_pspm(index).as_count_matrix();
-}
-
-function motif_prob_matrix(index) {
-  return motif_pspm(index).as_probability_matrix();
-}
-
-function motif_minimal_meme(index) {
-  return motif_pspm(index).as_meme({
-    "with_header": true, 
-    "with_pspm": true,
-    "with_pssm": true,
-    "version": data["version"],
-    "alphabet": meme_alphabet,
-    "strands": (meme_alphabet.has_complement() && data.options.revcomp ? 2 : 1)
-  });
-}
-
-function motif_fasta(index) {
-  "use strict";
-  var motif, sites, site, seq, sequences, sequence, i, num, counter, out;
-  counter = {};
-  sequences = data["sequence_db"]["sequences"];
-  motif = data["motifs"][index];
-  sites = motif["sites"];
-  out = "";
-  for (i = 0; i < sites.length; i++) {
-    site = sites[i];
-    seq = site["seq"];
-    sequence = sequences[seq];
-    counter[seq] = (num = counter[seq]) ? (++num) : (num = 1); // inc counter
-    if (i !== 0) {out += "\n";}
-    out += ">" + sequence["name"] + "_site_" + num + " offset= " + site["pos"] + 
-      (site["rc"] ? " RC\n" : "\n");
-    out += site["match"];
-  }
-  return out;
-}
-
-function motif_raw(index) {
-  "use strict";
-  var sites, i, out;
-  sites = data["motifs"][index]["sites"];
-  out = "";
-  for (i = 0; i < sites.length; i++) {
-    if (i !== 0) {out += "\n";}
-    out += sites[i]["match"];
-  }
-  return out;
-}
-
 function clone_template(template) {
   "use strict";
   var node, help_btns, i, button;
@@ -95,33 +30,6 @@ function clone_template(template) {
   return node;
 }
 
-function set_tvar(template, tvar, value) {
-  var node;
-  node = find_child(template, tvar);
-  if (node === null) {
-    throw new Error("Template does not contain variable " + tvar);
-  }
-  node.innerHTML = "";
-  if (typeof value !== "object") {
-    node.appendChild(document.createTextNode(value));
-  } else {
-    node.appendChild(value);
-  }
-}
-
-function make_logo(alphabet, pspm, rc, offset, className) {
-  if (rc) pspm = pspm.copy().reverse_complement(alphabet);
-  var logo = new Logo(alphabet, "");
-  logo.add_pspm(pspm, offset);
-  var canvas = document.createElement('canvas');
-  canvas.height = 50;
-  canvas.width = 0;
-  canvas.className = className;
-  size_logo_on_canvas(logo, canvas, false);
-  add_draw_task(canvas, new DelayLogoTask(logo, canvas));
-  return canvas;
-}
-
 function make_small_logo(alphabet, pspm, options) {
   if (typeof options === "undefined") options = {};
   if (options.rc) pspm = pspm.copy().reverse_complement(alphabet);
@@ -219,7 +127,7 @@ function append_site_entries(tbody, motif, site_index, count) {
     add_text_cell(rbody, site["pos"] + 1, "site_start");
     add_text_cell(rbody, site["pvalue"].toExponential(2), "site_pvalue");
     add_text_cell(rbody, site["lflank"], "site lflank");
-    add_cell(rbody, make_seq(meme_alphabet, site["match"]), "site match");
+    add_cell(rbody, make_seq(current_alphabet, site["match"]), "site match");
     add_text_cell(rbody, site["rflank"], "site rflank");
   }
   return i;
@@ -336,10 +244,10 @@ function make_motif_table_entry(row, alphabet, ordinal, motif, colw) {
       preview_rc = motif["preview_logo_rc"];
     } else {
       pspm = new Pspm(motif["pwm"]);
-      preview = make_logo(alphabet, pspm);
+      preview = make_logo(alphabet, pspm, 50, false, 0);
       motif["preview_logo"] = preview;
       if (alphabet.has_complement()) {
-        preview_rc = make_logo(alphabet, pspm, true, 0, "logo_rc");
+        preview_rc = make_logo(alphabet, pspm, 50, true, 0, "logo_rc");
         motif["preview_logo_rc"] = preview_rc;
       }
     }
@@ -381,6 +289,7 @@ function make_motif_table_entry(row, alphabet, ordinal, motif, colw) {
   }
   var pspm, preview, preview_rc, c;
   row.data_motif = motif;
+  row.id = motif["alt"];
   row.data_ordinal = ordinal;
   if (!ev_sig(motif["evalue"])) {
     row.style.opacity = 0.4;
@@ -391,12 +300,8 @@ function make_motif_table_entry(row, alphabet, ordinal, motif, colw) {
   add_text_cell(row, motif["nsites"], "motif_nsites");
   add_text_cell(row, motif["len"], "motif_width");
   add_cell(row, make_sym_btn("\u21A7", "Show more information.", 
-        action_show_more), "motif_more");
-  add_cell(row, 
-      make_sym_btn("\u21E2", 
-        "Submit the motif to another MEME Suite program or download it.",
-        action_show_outpop), 
-      "motif_submit");
+    action_show_more), "motif_more");
+  add_cell(row, make_sym_btn("\u21E2", "Submit the motif to another MEME Suite program or download it.", function(e) { action_show_outpop(e, ordinal); }), "motif_submit");
   if (colw) {
     for (c = 0; c < row.cells.length; c++) {
       row.cells[c].style.minWidth = colw[c] + "px";
@@ -420,7 +325,7 @@ function make_motifs_table(alphabet, start_ordinal, motifs, colw, stop_reason) {
 
   row = thead.insertRow(thead.rows.length);
   add_text_header_cell(row, "", "", "motif_ordinal");
-  add_text_header_cell(row, "Logo", "", "motif_logo");
+  add_text_header_cell(row, "Logo", "pop_logo", "motif_logo");
   add_text_header_cell(row, "E-value", "pop_ev", "motif_evalue");
   add_text_header_cell(row, "Sites", "pop_sites", "motif_nsites");
   add_text_header_cell(row, "Width", "pop_width", "motif_width");
@@ -512,10 +417,9 @@ function make_expanded_motif(alphabet, ordinal, motif, less_x, submit_x) {
   return box;
 }
 
-
 //
 //
-///
+//
 function make_motifs() {
   "use strict";
   function pixel_value(str_in) {
@@ -534,7 +438,7 @@ function make_motifs() {
   container = $("motifs");
   container.innerHTML = ""; // clear content
 
-  tbl = make_motifs_table(meme_alphabet, 1, data["motifs"], colw, data["stop_reason"]);
+  tbl = make_motifs_table(current_alphabet, 1, data["motifs"], colw, data["stop_reason"]);
   container.appendChild(tbl);
 
   // measure table column widths
@@ -597,7 +501,7 @@ function append_blocks_entries(tbody, seq_index, count) {
     // get the sequence
     sequence = data.sequence_db.sequences[i];
     // make the containers for the block diagram
-    container = make_block_container(meme_alphabet.has_complement(),
+    container = make_block_container(current_alphabet.has_complement(),
         data.options.revcomp, max_seq_len, sequence.length);
     // create blocks for the motif sites
     sites = sequence["sites"];
@@ -770,7 +674,7 @@ var downloadBlockDiagram = function(
   var max_pvalue_width = MeasureText(font, "p-value");
 
   var max_seq_length = 0;		// in characters
-  var has_complement = meme_alphabet.has_complement();
+  var has_complement = current_alphabet.has_complement();
   var revcomp = data.options["revcomp"];
 
   // Measure text of numbers, names and p-values, convert to points and save the max.
@@ -1233,7 +1137,7 @@ function make_blocks() {
     }
   }
   function add_block_diagram_button(container, id, buttonText, help_topic) {
-    var button, label, new_icon;
+    var button, label;
     button = document.createElement("button");
     button.id = id;
     label = document.createTextNode(buttonText);
@@ -1245,11 +1149,11 @@ function make_blocks() {
       container.appendChild(document.createTextNode("\xA0"));
       container.appendChild(help_button(help_topic));
     }
-    new_icon = document.createElement("img");
-    new_icon.src = new_icon_src;
-    new_icon.alt = "NEW";
-    container.appendChild(document.createTextNode("  "));
-    container.appendChild(new_icon);
+    //var new_icon = document.createElement("img");
+    //new_icon.src = new_icon_src;
+    //new_icon.alt = "NEW";
+    //container.appendChild(document.createTextNode("  "));
+    //container.appendChild(new_icon);
   }
   function add_blocks_header(row, title, nopad, help_topic) {
     "use strict";
@@ -1341,8 +1245,8 @@ function make_scan_popup(site, motif) {
       if (pop) return;
       pop = clone_template("tmpl_scan_info");
       pspm = new Pspm(motif.pwm);
-      if (site.rc) pspm.reverse_complement(meme_alphabet);
-      set_tvar(pop, "tvar_logo", make_small_logo(meme_alphabet, pspm, {"className": "scan_logo"}));
+      if (site.rc) pspm.reverse_complement(current_alphabet);
+      set_tvar(pop, "tvar_logo", make_small_logo(current_alphabet, pspm, {"className": "scan_logo"}));
       set_tvar(pop, "tvar_motif", motif.id);
       set_tvar(pop, "tvar_pvalue", site.pvalue.toExponential(2));
       set_tvar(pop, "tvar_start", site.pos + 1);
@@ -1372,17 +1276,17 @@ function make_block_popup(site, motif, block) {
       pop = clone_template("tmpl_block_info");
       pspm = new Pspm(motif.pwm);
       if (site.rc) { // must be dna
-        pspm.reverse_complement(meme_alphabet);
-        lflank = meme_alphabet.invcomp_seq(site.rflank);
-        match = meme_alphabet.invcomp_seq(site.match);
-        rflank = meme_alphabet.invcomp_seq(site.lflank);
+        pspm.reverse_complement(current_alphabet);
+        lflank = current_alphabet.invcomp_seq(site.rflank);
+        match = current_alphabet.invcomp_seq(site.match);
+        rflank = current_alphabet.invcomp_seq(site.lflank);
       } else {
         lflank = site.lflank;
         match = site.match;
         rflank = site.rflank;
       }
       ruler = document.getElementById("measure_match");
-      match_seq = make_seq(meme_alphabet, match);
+      match_seq = make_seq(current_alphabet, match);
       ruler.innerHTML = "";
       ruler.appendChild(match_seq);
       match_width = ruler.clientWidth;
@@ -1391,7 +1295,7 @@ function make_block_popup(site, motif, block) {
       set_tvar(pop, "tvar_match", match_seq);
       set_tvar(pop, "tvar_rflank", rflank);
       set_tvar(pop, "tvar_logo_pad", lflank);
-      set_tvar(pop, "tvar_logo", make_small_logo(meme_alphabet, pspm, {"width": match_width}));
+      set_tvar(pop, "tvar_logo", make_small_logo(current_alphabet, pspm, {"width": match_width}));
       set_tvar(pop, "tvar_motif", motif.id);
       set_tvar(pop, "tvar_pvalue", site.pvalue.toExponential(2));
       set_tvar(pop, "tvar_start", site.pos + 1);
@@ -1409,59 +1313,6 @@ function make_block_popup(site, motif, block) {
   };
 }
 
-function update_outpop_format(index) {
-  switch(parseInt($("text_format").value)) {
-    case 0: // count matrix
-      $("outpop_text").value = motif_count_matrix(index);
-      $("text_name").value = "motif_" + (index + 1) + "_counts.txt";
-      break;
-    case 1: // prob matrix
-      $("outpop_text").value = motif_prob_matrix(index);
-      $("text_name").value = "motif_" + (index + 1) + "_freqs.txt";
-      break;
-    case 2: // minimal meme
-      $("outpop_text").value = motif_minimal_meme(index);
-      $("text_name").value = "motif_" + (index + 1) + ".txt";
-      break;
-    case 3: // fasta
-      $("outpop_text").value = motif_fasta(index);
-      $("text_name").value = "motif_" + (index + 1) + "_fasta.txt";
-      break;
-    case 4: // raw
-      $("outpop_text").value = motif_raw(index);
-      $("text_name").value = "motif_" + (index + 1) + "_raw.txt";
-      break;
-    default:
-      throw new Error("Unknown motif format");
-  }
-}
-
-function update_outpop_motif(index) {
-  "use strict";
-  var motifs, motif, pspm, logo, canvas, num;
-  motifs = data["motifs"];
-  if (index < 0 || index >= motifs.length) {return;}
-  current_motif = index;
-  motif = motifs[index];
-  pspm = new Pspm(motif["pwm"]);
-  logo = new Logo(meme_alphabet, "");
-  logo.add_pspm(pspm, 0);
-  canvas = $("outpop_logo");
-  canvas.width = canvas.width; // clear canvas
-  draw_logo_on_canvas(logo, canvas, false);
-  if (meme_alphabet.has_complement()) {
-    pspm.reverse_complement(meme_alphabet);
-    logo = new Logo(meme_alphabet, "");
-    canvas = $("outpop_logo_rc");
-    canvas.width = canvas.width; // clear canvas
-    draw_logo_on_canvas(logo, canvas, false);
-  }
-  num = $("outpop_num");
-  num.innerHTML = "";
-  num.appendChild(document.createTextNode("" + (index + 1)));
-  update_outpop_format(index);
-}
-
 //
 // action_show_more
 //
@@ -1504,7 +1355,7 @@ function action_show_more(e) {
   container = node.parentNode;
   // make a expanded motif
   motif["expanded"] = true;
-  expanded_motif = make_expanded_motif(meme_alphabet, ordinal, motif, 
+  expanded_motif = make_expanded_motif(current_alphabet, ordinal, motif, 
       container.data_more_x, container.data_submit_x);
   // now determine how to place it
   if (tbody.rows.length === 1) {
@@ -1582,7 +1433,7 @@ function action_show_less(e) {
   if (table_before && table_after) {
     tbody = table_before.tBodies[0];
     row = tbody.insertRow(tbody.rows.length);
-    make_motif_table_entry(row, meme_alphabet, ordinal, motif, colw);
+    make_motif_table_entry(row, current_alphabet, ordinal, motif, colw);
     focus_target = find_child(row.cells[5], "sym_btn");
     container.removeChild(expanded_motif);
     tbody2 = table_after.tBodies[0];
@@ -1595,26 +1446,28 @@ function action_show_less(e) {
   } else if (table_before) {
     tbody = table_before.tBodies[0];
     row = tbody.insertRow(tbody.rows.length);
-    make_motif_table_entry(row, meme_alphabet, ordinal, motif, colw);
+    make_motif_table_entry(row, current_alphabet, ordinal, motif, colw);
     focus_target = find_child(row.cells[5], "sym_btn");
     container.removeChild(expanded_motif);
   } else if (table_after) {
     tbody = table_after.tBodies[0];
     row = tbody.insertRow(0);
-    make_motif_table_entry(row, meme_alphabet, ordinal, motif, colw);
+    make_motif_table_entry(row, current_alphabet, ordinal, motif, colw);
     focus_target = find_child(row.cells[5], "sym_btn");
     container.removeChild(expanded_motif);
   } else {
     //no table above or below!
     // make a new table
-    table = make_motifs_table(meme_alphabet, ordinal, [motif], colw, data["stop_reason"]);
+    table = make_motifs_table(current_alphabet, ordinal, [motif], colw, data["stop_reason"]);
     focus_target = find_child(table.tBodies[0].rows[0].cells[5], "sym_btn");
     container.replaceChild(table, expanded_motif);
   }
   focus_target.focus();
 }
 
-function action_show_outpop(e) {
+//FIXME -- can we delete this junk?
+//function action_show_outpop(e) {
+function fred_action_show_outpop(e) {
   "use strict";
   function init() {
     "use strict";
@@ -1662,18 +1515,18 @@ function action_show_outpop(e) {
     // setup program selection
     tbl_submit = $("programs");
     // when not dna, hide the inputs for programs that require dna motifs
-    toggle_class(tbl_submit, "alphabet_dna", meme_alphabet.has_complement());//TODO FIXME alphabet_dna is a bad name for a field when allowing custom alphabets
+    toggle_class(tbl_submit, "alphabet_dna", current_alphabet.has_complement());//TODO FIXME alphabet_dna is a bad name for a field when allowing custom alphabets
     // add a click listener for the radio buttons
     inputs = tbl_submit.querySelectorAll("input[type='radio']");
     for (i = 0; i < inputs.length; i++) {
       inputs[i].addEventListener("click", action_outpop_program, false);
     }
     // ensure that a default program option is selected for DNA and Protein
-    default_prog = document.getElementById(meme_alphabet.has_complement() ? "submit_tomtom" : "submit_fimo"); //TODO FIXME Tomtom might require a more strict definition of DNA
+    default_prog = document.getElementById(current_alphabet.has_complement() ? "submit_tomtom" : "submit_fimo"); //TODO FIXME Tomtom might require a more strict definition of DNA
     default_prog.checked = true;
     action_outpop_program.call(default_prog);
     // disable reverse-complement when not DNA
-    $("logo_rc_option").disabled = !meme_alphabet.has_complement(); 
+    $("logo_rc_option").disabled = !current_alphabet.has_complement(); 
     // set errorbars on when ssc is on
     $("logo_ssc").addEventListener("change", action_outpop_ssc, false);
   }
@@ -1706,100 +1559,7 @@ function action_show_outpop(e) {
   $("grey_out_page").style.display = "block";
   $("download").style.display = "block";
   $("outpop_close").focus();
-}
-
-function action_hide_outpop(e) {
-  if (!e) e = window.event;
-  if (e.type === "keydown") {
-    if (e.keyCode !== 13 && e.keyCode !== 32) {
-      return;
-    }
-    // stop a submit or something like that
-    e.preventDefault();
-  }
-  $("download").style.display = "none";
-  $("grey_out_page").style.display = "none";
-  if (typeof action_hide_outpop.last_active !== "undefined") {
-    action_hide_outpop.last_active.focus();
-  }
-}
-
-function action_outpop_next(e) {
-  if (!e) e = window.event;
-  if (e.type === "keydown") {
-    if (e.keyCode !== 13 && e.keyCode !== 32) {
-      return;
-    }
-    // stop a submit or something like that
-    e.preventDefault();
-  }
-  update_outpop_motif(current_motif + 1);
-}
-
-function action_outpop_prev(e) {
-  if (!e) e = window.event;
-  if (e.type === "keydown") {
-    if (e.keyCode !== 13 && e.keyCode !== 32) {
-      return;
-    }
-    // stop a submit or something like that
-    e.preventDefault();
-  }
-  update_outpop_motif(current_motif - 1);
-}
-
-function action_outpop_program() {
-  "use strict";
-  var table, tr, rows, i;
-  tr = find_parent_tag(this, "TR");
-  table = find_parent_tag(tr, "TABLE");
-  rows = table.querySelectorAll("tr");
-  for (i = 0; i < rows.length; i++) {
-    toggle_class(rows[i], "selected", rows[i] === tr);
-  }
-}
-
-function action_outpop_ssc() {
-  "use strict";
-  $("logo_err").value = $("logo_ssc").value;
-}
-
-function action_outpop_submit(e) {
-  "use strict";
-  var form, input, program, motifs;
-  // find out which program is selected
-  var radios, i;
-  radios = document.getElementsByName("program");
-  program = "fimo"; // default to fimo, since it works with all alphabet types
-  for (i = 0; i < radios.length; i++) {
-    if (radios[i].checked) program = radios[i].value;
-  }
-
-  motifs = motif_minimal_meme(current_motif);
-  form = document.createElement("form");
-  form.setAttribute("method", "post");
-  form.setAttribute("action", site_url + "/tools/" + program);
-  
-  input = document.createElement("input");
-  input.setAttribute("type", "hidden");
-  input.setAttribute("name", "motifs_embed");
-  input.setAttribute("value", motifs);
-  form.appendChild(input);
-
-  document.body.appendChild(form);
-  form.submit();
-  document.body.removeChild(form);
-}
-
-function action_outpop_download_motif(e) {
-  $("text_form").submit();
-}
-
-function action_outpop_download_logo(e) {
-  "use strict";
-  $("logo_motifs").value = motif_minimal_meme(current_motif);
-  $("logo_form").submit();
-}
+} // fred_action_show_outpop
 
 function action_btn_rc(e) {
   "use strict";
@@ -1853,44 +1613,6 @@ function action_rc_tab(e) {
   toggle_class(tab_rc, "activeTab", rc);
 }
 
-function action_outpop_tab(e) {
-  "use strict";
-  var tab1, tab2, tab3, pnl1, pnl2, pnl3, do_btn;
-  if (!e) e = window.event;
-  if (e.type === "keydown") {
-    if (e.keyCode !== 13 && e.keyCode !== 32) {
-      return;
-    }
-    // stop a submit or something like that
-    e.preventDefault();
-  }
-  tab1 = $("outpop_tab_1");
-  tab2 = $("outpop_tab_2");
-  tab3 = $("outpop_tab_3");
-  pnl1 = $("outpop_pnl_1");
-  pnl2 = $("outpop_pnl_2");
-  pnl3 = $("outpop_pnl_3");
-  do_btn = $("outpop_do");
-
-  toggle_class(tab1, "activeTab", (this === tab1));
-  toggle_class(tab2, "activeTab", (this === tab2));
-  toggle_class(tab3, "activeTab", (this === tab3));
-  pnl1.style.display = ((this === tab1) ? "block" : "none");
-  pnl2.style.display = ((this === tab2) ? "block" : "none");
-  pnl3.style.display = ((this === tab3) ? "block" : "none");
-  do_btn.value = ((this === tab1) ? "Submit" : "Download");
-  do_btn.removeEventListener("click", action_outpop_submit, false);
-  do_btn.removeEventListener("click", action_outpop_download_logo, false);
-  do_btn.removeEventListener("click", action_outpop_download_motif, false);
-  if (this === tab1) {
-    do_btn.addEventListener("click", action_outpop_submit, false);
-  } else if (this === tab2) {
-    do_btn.addEventListener("click", action_outpop_download_motif, false);
-  } else {
-    do_btn.addEventListener("click", action_outpop_download_logo, false);
-  }
-}
-
 function action_seqs_filter() {
   "use strict";
   var block_container;
@@ -1907,10 +1629,6 @@ function action_seqs_filter() {
   }
 }
 
-function action_outpop_format() {
-  update_outpop_format(current_motif);
-}
-
 //
 // page_loaded
 //
