@@ -61,13 +61,13 @@ void print_matrices(
 #ifndef VITERBI_DEBUG
 #define VITERBI_DEBUG 0
 #endif
-BOOLEAN_T viterbi_algorithm
-  (BOOLEAN_T local,         // Perform local scoring? (else global)
+bool viterbi_algorithm
+  (bool local,         // Perform local scoring? (else global)
    int*      int_sequence,  // The sequence to be scored, int format.
    int       seq_length,    // The length of the sequence.
    MHMM_T*   the_log_hmm,   // An HMM in which the distributions have
                             // been converted to scaled log format.
-   BOOLEAN_T compute_trace, // Save Viterbi path?
+   bool compute_trace, // Save Viterbi path?
    MATRIX_T* motif_score_matrix, // Position-specific scores for each motif.
    MATRIX_T* dp_matrix,     // Dynamic programming matrix.
    MATRIX_T* trace_matrix,  // Traceback matrix.
@@ -77,7 +77,7 @@ BOOLEAN_T viterbi_algorithm
   int       i_row;           // Index of current row in dp matrix.
   int       i_col;           // Index of current column in dp matrix.
   const int num_states = the_log_hmm->num_states;
-  BOOLEAN_T motif_scoring;
+  bool motif_scoring;
   int       num_hot_states = the_log_hmm->num_hot_states;
   int*       hot_states = the_log_hmm->hot_states;
   double    best_score;
@@ -215,7 +215,7 @@ BOOLEAN_T viterbi_algorithm
 #define FORWARD_DEBUG 0
 #endif
 void forward_algorithm
-  (BOOLEAN_T local,         // Perform local scoring? (else global)
+  (bool local,         // Perform local scoring? (else global)
    int*      int_sequence,  // Sequence to be scored, int format.
    int       seq_length,    // The length of the sequence.
    MHMM_T *  the_log_hmm,   // An HMM in log format.
@@ -226,7 +226,7 @@ void forward_algorithm
   int       i_col;   // Index of current column in dp matrix.
   const int num_states = the_log_hmm->num_states; // Number of states in HMM.
   //FIXME: handle motif scoring here!
-  BOOLEAN_T motif_scoring = FALSE;
+  bool motif_scoring = false;
 
   // Initialize the first column to zeroes.
   for (i_row = 1; i_row < num_states; i_row++) {
@@ -335,7 +335,7 @@ void forward_algorithm
   // Compute the sum of the last column.
   traceback_and_scoring(
     local ? LOCAL_TOTAL_SEARCH : GLOBAL_TOTAL_SEARCH,
-    FALSE, // Don't compute traceback.
+    false, // Don't compute traceback.
     num_states,
     seq_length,
     the_log_hmm,
@@ -452,7 +452,7 @@ void backward_algorithm
  * same for each state.  This function verifies that equality.
  **************************************************************************/
 #define SLOP 1E-6
-BOOLEAN_T verify_matrices
+bool verify_matrices
   (int       num_states,
    int       max_length, 
    PROB_T    forward_score,
@@ -480,10 +480,10 @@ BOOLEAN_T verify_matrices
         "Matrix error for position %d: difference=%.4g.\n", i_seq,
         forward_score - this_total);
       abort();
-      return(FALSE);
+      return(false);
     }
   }
-  return(TRUE);
+  return(true);
 }
 
 /**************************************************************************
@@ -520,7 +520,7 @@ void repeated_match_algorithm
   int       i_row;           // Index of current row in dp matrix.
   int       i_col;           // Index of current column in dp matrix.
   int       in_index;
-  BOOLEAN_T motif_scoring = (motif_score_matrix != NULL);
+  bool motif_scoring = (motif_score_matrix != NULL);
   int       num_hot_states = the_log_hmm->num_hot_states;
   int*       hot_states = the_log_hmm->hot_states;
   const int num_states = the_log_hmm->num_states;
@@ -685,7 +685,7 @@ void repeated_match_algorithm
   // Traceback and scoring.
   traceback_and_scoring(
     REPEATED_MATCH_SEARCH,
-    TRUE,   // Compute the trace.
+    true,   // Compute the trace.
     num_states,
     seq_length,
     the_log_hmm,
