@@ -18,7 +18,6 @@
 #include "utils.h"
 
 // Nucleotide alphabest order as in motif.h
-extern char alphabet[];
 
 static char * model_names[] = { 
   "single",
@@ -32,6 +31,7 @@ static char * model_names[] = {
 };
 
 static  MATRIX_T* make_rate_matrix(
+  ALPH_T* dna_alph,
   ARRAY_T* equil_freqs,
   const double non_specific_rate,
   const double pur_pur_rate,
@@ -85,8 +85,9 @@ static double tamura_nei_evaluator(
   * nucleotide substitution
   ****************************************************************************/
 struct evomodel {
+  ALPH_T*        alph;
   MODEL_TYPE_T   type;
-  BOOLEAN_T      use_halpern_bruno;
+  bool      use_halpern_bruno;
   int            num_params;   // Number of model parameters
   char**         param_names;  // Names of the parameters, useful for debugging
   double*        param_values;
@@ -108,8 +109,8 @@ struct evomodel {
   ****************************************************************************/
 void print_evomodel(EVOMODEL_T* model)
 {
-  print_array(model->equil_freqs, 8, 4, TRUE, stdout);
-  print_matrix(model->rate_matrix, 8, 4, TRUE, stdout);
+  print_array(model->equil_freqs, 8, 4, true, stdout);
+  print_matrix(model->rate_matrix, 8, 4, true, stdout);
 }
 
 /*******************************************************************
@@ -124,11 +125,11 @@ EVOMODEL_T** make_motif_models(
   double bg_rate,
   double purine_pyrimidine,
   double transition_transversion,
-  BOOLEAN_T use_halpern_bruno
+  bool use_halpern_bruno
 ) 
 {
-  int alph_size = get_alph_size(ALPH_SIZE);
-  int num_models = motif->length + 1;
+  int asize = alph_size_core(get_motif_alph(motif));
+  int num_models = get_motif_length(motif) + 1;
   EVOMODEL_T** models = mm_malloc(num_models * sizeof(EVOMODEL_T*));
 
   // Make the background model and save it as models[0].
@@ -149,13 +150,13 @@ EVOMODEL_T** make_motif_models(
 
   for (i = 1; i < num_models; i++) {
     // Use motif PSFM for equilibrium freqs. for model.
-    ARRAY_T* site_specific_freqs = allocate_array(alph_size);
+    ARRAY_T* site_specific_freqs = allocate_array(asize);
     int j = 0;
-    for(j = 0; j < alph_size; j++) {
-      double value = get_matrix_cell(i - 1, j, motif->freqs);
+    for(j = 0; j < asize; j++) {
+      double value = get_matrix_cell(i - 1, j, get_motif_freqs(motif));
       set_array_item(j, value, site_specific_freqs);
     }
-    if (use_halpern_bruno == FALSE) {
+    if (use_halpern_bruno == false) {
       // If not using Halpern-Bruno use site specific freqs for 
       // equilibrium freqs.
       model = make_model(
@@ -267,10 +268,11 @@ EVOMODEL_T* make_model(
 EVOMODEL_T* make_psfm_model(ARRAY_T* equil_freqs) {
 
   EVOMODEL_T* model = mm_malloc(sizeof(EVOMODEL_T));
+  model->alph = alph_dna();
   model->type = SINGLE_MODEL;
   model->equil_freqs = allocate_array(get_array_length(equil_freqs));
   copy_array(equil_freqs, model->equil_freqs);
-  model->use_halpern_bruno = FALSE;
+  model->use_halpern_bruno = false;
 
   // No parameters for the SINGLE_MODEL
   model->num_params = 0;
@@ -290,10 +292,11 @@ EVOMODEL_T* make_avg_psfm_model (ARRAY_T* equil_freqs) {
   EVOMODEL_T* model = NULL;
 
   model = mm_malloc(sizeof(EVOMODEL_T));
+  model->alph = alph_dna();
   model->type = AVERAGE_MODEL;
   model->equil_freqs = allocate_array(get_array_length(equil_freqs));
   copy_array(equil_freqs, model->equil_freqs);
-  model->use_halpern_bruno = FALSE;
+  model->use_halpern_bruno = false;
 
   // No parameters for the AVERAGE_MODEL
   model->num_params = 0;
@@ -320,10 +323,11 @@ EVOMODEL_T* make_jc_model (
   ARRAY_T* jc_freqs = NULL;
 
   model = mm_malloc(sizeof(EVOMODEL_T));
+  model->alph = alph_dna();
   model->type = JC_MODEL;
   model->equil_freqs = allocate_array(get_array_length(equil_freqs));
   copy_array(equil_freqs, model->equil_freqs);
-  model->use_halpern_bruno = FALSE;
+  model->use_halpern_bruno = false;
   model->evaluator = jc_evaluator;
 
   // The only parameter for Jukes-Cantor is the subsitution substitution_rate
@@ -341,7 +345,7 @@ EVOMODEL_T* make_jc_model (
   }
 
   // Make rate matrix from equivalent Tamura-Nei model.
-  model->rate_matrix = make_rate_matrix(jc_freqs, 4 * rate / 3, 0.0, 0.0);
+  model->rate_matrix = make_rate_matrix(model->alph, jc_freqs, 4 * rate / 3, 0.0, 0.0);
   free_array(jc_freqs);
 
   return model;
@@ -361,10 +365,11 @@ EVOMODEL_T* make_k2_model(
 ) 
 {
   EVOMODEL_T* model = mm_malloc(sizeof(EVOMODEL_T));
+  model->alph = alph_dna();
   model->type = K2_MODEL;
   model->equil_freqs = allocate_array(get_array_length(equil_freqs));
   copy_array(equil_freqs, model->equil_freqs);
-  model->use_halpern_bruno = FALSE;
+  model->use_halpern_bruno = false;
   model->evaluator = k2_evaluator;
 
   // The only parameters for the Kimura 2 parameter model
@@ -399,6 +404,7 @@ EVOMODEL_T* make_k2_model(
     / ( 2.0 * Pr * Py * (1.0 + transition_transversion));
   // Build rate matrix based on equivalent Tamura-Nei model
   model->rate_matrix = make_rate_matrix(
+    model->alph, 
     k2_freqs,
     non_specific_rate,
     pur_pur_rate, 
@@ -420,10 +426,11 @@ EVOMODEL_T* make_f81_model(
 ) 
 {
   EVOMODEL_T* model = mm_malloc(sizeof(EVOMODEL_T));
+  model->alph = alph_dna();
   model->type = F81_MODEL;
   model->equil_freqs = allocate_array(get_array_length(equil_freqs));
   copy_array(equil_freqs, model->equil_freqs);
-  model->use_halpern_bruno = FALSE;
+  model->use_halpern_bruno = false;
   model->evaluator = f81_evaluator;
 
   // The parameters for the F81 model are the equilibrium base frequencies.
@@ -436,10 +443,11 @@ EVOMODEL_T* make_f81_model(
   (model->param_names)[4] = "T frequency";
   model->param_values = (double *) mm_malloc(model->num_params * sizeof(double));
   (model->param_values)[0] = rate;
-  double Pa = get_array_item(alphabet_index('A', alphabet), model->equil_freqs);
-  double Pc = get_array_item(alphabet_index('C', alphabet), model->equil_freqs);
-  double Pg = get_array_item(alphabet_index('G', alphabet), model->equil_freqs);
-  double Pt = get_array_item(alphabet_index('T', alphabet), model->equil_freqs);
+  // TODO investigate alphabet use, hardcoding of symbols should be avoided if possible
+  double Pa = get_array_item(alph_index(model->alph, 'A'), model->equil_freqs);
+  double Pc = get_array_item(alph_index(model->alph, 'C'), model->equil_freqs);
+  double Pg = get_array_item(alph_index(model->alph, 'G'), model->equil_freqs);
+  double Pt = get_array_item(alph_index(model->alph, 'T'), model->equil_freqs);
   double Pr = Pa + Pg;
   double Py = Pc + Pt;
   (model->param_values)[1] = Pa;
@@ -454,6 +462,7 @@ EVOMODEL_T* make_f81_model(
 
   // Build rate matrix from equivalent Tamura-Nei model
   model->rate_matrix = make_rate_matrix(
+    model->alph, 
     model->equil_freqs, 
     non_specific_rate,
     pyr_pyr_rate, 
@@ -476,10 +485,11 @@ EVOMODEL_T* make_f84_model(
 ) 
 {
   EVOMODEL_T* model = mm_malloc(sizeof(EVOMODEL_T));
+  model->alph = alph_dna();
   model->type = F84_MODEL;
   model->equil_freqs = allocate_array(get_array_length(equil_freqs));
   copy_array(equil_freqs, model->equil_freqs);
-  model->use_halpern_bruno = FALSE;
+  model->use_halpern_bruno = false;
   model->evaluator = f84_evaluator;
 
   // The parameters for the F84 model are the equilibrium 
@@ -494,10 +504,11 @@ EVOMODEL_T* make_f84_model(
   (model->param_names)[5] = "ratio of transition/transversion rates ";
   model->param_values = (double *) mm_malloc(model->num_params * sizeof(double));
   (model->param_values)[0] = rate;
-  double Pa = get_array_item(alphabet_index('A', alphabet), model->equil_freqs);
-  double Pc = get_array_item(alphabet_index('C', alphabet), model->equil_freqs);
-  double Pg = get_array_item(alphabet_index('G', alphabet), model->equil_freqs);
-  double Pt = get_array_item(alphabet_index('T', alphabet), model->equil_freqs);
+  // TODO investigate further, hard coding of symbols should be avoided if possible
+  double Pa = get_array_item(alph_index(model->alph, 'A'), model->equil_freqs);
+  double Pc = get_array_item(alph_index(model->alph, 'C'), model->equil_freqs);
+  double Pg = get_array_item(alph_index(model->alph, 'G'), model->equil_freqs);
+  double Pt = get_array_item(alph_index(model->alph, 'T'), model->equil_freqs);
   (model->param_values)[1] = Pa;
   (model->param_values)[2] = Pc;
   (model->param_values)[3] = Pg;
@@ -514,6 +525,7 @@ EVOMODEL_T* make_f84_model(
   double non_specific_rate = 
     rate / ( 2.0 * Pr * Py * (1.0 + transition_transversion));
   model->rate_matrix = make_rate_matrix(
+    model->alph, 
     model->equil_freqs,
     non_specific_rate,
     pur_pur_rate, 
@@ -536,10 +548,11 @@ EVOMODEL_T* make_hky_model(
 ) 
 {
   EVOMODEL_T* model = mm_malloc(sizeof(EVOMODEL_T));
+  model->alph = alph_dna();
   model->type = HKY_MODEL;
   model->equil_freqs = allocate_array(get_array_length(equil_freqs));
   copy_array(equil_freqs, model->equil_freqs);
-  model->use_halpern_bruno = FALSE;
+  model->use_halpern_bruno = false;
   model->evaluator = hky_evaluator;
 
   // The parameters for the HKY model are the equilibrium 
@@ -556,10 +569,11 @@ EVOMODEL_T* make_hky_model(
     " rates";
   model->param_values = (double *) mm_malloc(model->num_params * sizeof(double));
   (model->param_values)[0] = rate;
-  double Pa = get_array_item(alphabet_index('A', alphabet), model->equil_freqs);
-  double Pc = get_array_item(alphabet_index('C', alphabet), model->equil_freqs);
-  double Pg = get_array_item(alphabet_index('G', alphabet), model->equil_freqs);
-  double Pt = get_array_item(alphabet_index('T', alphabet), model->equil_freqs);
+  // TODO investigate further, hardcoding of symbols should be avoided if possible
+  double Pa = get_array_item(alph_index(model->alph, 'A'), model->equil_freqs);
+  double Pc = get_array_item(alph_index(model->alph, 'C'), model->equil_freqs);
+  double Pg = get_array_item(alph_index(model->alph, 'G'), model->equil_freqs);
+  double Pt = get_array_item(alph_index(model->alph, 'T'), model->equil_freqs);
   (model->param_values)[1] = Pa;
   (model->param_values)[2] = Pc;
   (model->param_values)[3] = Pg;
@@ -576,6 +590,7 @@ EVOMODEL_T* make_hky_model(
   double non_specific_rate = 
     rate / ( 2.0 * Pr * Py * (1.0 + transition_transversion));
   model->rate_matrix = make_rate_matrix(
+    model->alph, 
     model->equil_freqs,
     non_specific_rate,
     pur_pur_rate, 
@@ -599,10 +614,11 @@ EVOMODEL_T* make_tamura_nei_model(
   const double purine_pyrimidine) 
 {
   EVOMODEL_T* model = mm_malloc(sizeof(EVOMODEL_T));
+  model->alph = alph_dna();
   model->type = TAMURA_NEI_MODEL;
   model->equil_freqs = allocate_array(get_array_length(equil_freqs));
   copy_array(equil_freqs, model->equil_freqs);
-  model->use_halpern_bruno = FALSE;
+  model->use_halpern_bruno = false;
   model->evaluator = tamura_nei_evaluator;
 
   // The parameters for the Tamura-Nei model are the equilibrium 
@@ -620,10 +636,11 @@ EVOMODEL_T* make_tamura_nei_model(
   (model->param_names)[6] = "ratio of R/R to Y/Y transitions";
   model->param_values = (double *) mm_malloc(model->num_params * sizeof(double));
   (model->param_values)[0] = rate;
-  double Pa = get_array_item(alphabet_index('A', alphabet), model->equil_freqs);
-  double Pc = get_array_item(alphabet_index('C', alphabet), model->equil_freqs);
-  double Pg = get_array_item(alphabet_index('G', alphabet), model->equil_freqs);
-  double Pt = get_array_item(alphabet_index('T', alphabet), model->equil_freqs);
+  // TODO investigate further, hardcoding of symbols should be avoided
+  double Pa = get_array_item(alph_index(model->alph, 'A'), model->equil_freqs);
+  double Pc = get_array_item(alph_index(model->alph, 'C'), model->equil_freqs);
+  double Pg = get_array_item(alph_index(model->alph, 'G'), model->equil_freqs);
+  double Pt = get_array_item(alph_index(model->alph, 'T'), model->equil_freqs);
   (model->param_values)[1] = Pa;
   (model->param_values)[2] = Pc;
   (model->param_values)[3] = Pg;
@@ -642,6 +659,7 @@ EVOMODEL_T* make_tamura_nei_model(
   double non_specific_rate = 
     rate / ( 2.0 * Pr * Py * (1.0 + transition_transversion));
   model->rate_matrix = make_rate_matrix(
+    model->alph, 
     model->equil_freqs,
     non_specific_rate,
     pur_pur_rate, 
@@ -698,7 +716,7 @@ void make_halpern_bruno_model(EVOMODEL_T* model, ARRAY_T* site_specific_freqs) {
     }
   }
   model->rate_matrix = hb_matrix;
-  model->use_halpern_bruno = TRUE;
+  model->use_halpern_bruno = true;
   // Now replace the equilibrium frequecies with the site-specific ones
   // so that pruning algorithm will work.
   copy_array(site_specific_freqs, model->equil_freqs);
@@ -718,6 +736,7 @@ void free_model(EVOMODEL_T* model) {
   myfree(model->param_names);
   myfree(model->param_values);
   free_matrix(model->rate_matrix);
+  alph_release(model->alph);
   myfree(model);
 }
 
@@ -761,13 +780,14 @@ double* get_model_param_values(const EVOMODEL_T* model) {
 
 double get_model_equil_freq(char base, const EVOMODEL_T* model) {
 
-  double value = NaN();
+  double value = NAN;
+  int aindex = alph_index(model->alph, base);
 
   assert(model != NULL);
-  assert(strchr(alphabet, base));
+  assert(aindex != -1 && aindex < alph_size_core(model->alph));
 
   base = toupper(base);
-  value = get_array_item(alphabet_index(base, alphabet), model->equil_freqs);
+  value = get_array_item(aindex, model->equil_freqs);
 
   return value;
 };
@@ -798,23 +818,23 @@ MATRIX_T* get_model_rate_matrix(EVOMODEL_T* model) {
   return rate_matrix;
 }
 
-MATRIX_T* get_model_prob_matrix(double t, const EVOMODEL_T* model) {
+MATRIX_T* get_model_prob_matrix(ALPH_T* alph, double t, const EVOMODEL_T* model) {
 
   assert(model != NULL);
 
   MATRIX_T* prob_matrix = NULL;
 
-  if (model->use_halpern_bruno == TRUE) {
+  if (model->use_halpern_bruno == true) {
     prob_matrix = matrix_exponential(t, model->rate_matrix);
   } else {
-    int alph_size = get_alph_size(ALPH_SIZE);
+    int asize = alph_size_core(alph);
     prob_matrix = allocate_matrix(4, 4);
     int i;
-    for (i = 0; i < alph_size; i++) {
-      char old_base = alphabet[i];
+    for (i = 0; i < asize; i++) {
+      char old_base = alph_char(alph, i);
       int j;
-      for (j = 0; j < alph_size; j++) {
-        char new_base = alphabet[j];
+      for (j = 0; j < asize; j++) {
+        char new_base = alph_char(alph, j);
         // Calculate the probability of the old_base mutating
         // to the new_base in time t.
         double value = model->evaluator(old_base, new_base, t, model);
@@ -825,23 +845,23 @@ MATRIX_T* get_model_prob_matrix(double t, const EVOMODEL_T* model) {
   return prob_matrix;
 }
 
-MATRIX_T* old_get_model_prob_matrix(const EVOMODEL_T* model, double t) {
+MATRIX_T* old_get_model_prob_matrix(ALPH_T* alph, const EVOMODEL_T* model, double t) {
 
   assert(model != NULL);
 
   MATRIX_T* prob_matrix = NULL;
 
-  if (model->use_halpern_bruno == TRUE) {
+  if (model->use_halpern_bruno == true) {
     prob_matrix = matrix_exponential(t, model->rate_matrix);
   } else {
-    int alph_size = get_alph_size(ALPH_SIZE);
+    int asize = alph_size_core(alph);
     prob_matrix = allocate_matrix(4, 4);
     int i;
-    for (i = 0; i < alph_size; i++) {
-      char old_base = alphabet[i];
+    for (i = 0; i < asize; i++) {
+      char old_base = alph_char(alph, i);
       int j;
-      for (j = 0; j < alph_size; j++) {
-        char new_base = alphabet[j];
+      for (j = 0; j < asize; j++) {
+        char new_base = alph_char(alph, j);
         // Calculate the probability of the old_base mutating
         // to the new_base in time t.
         double value = model->evaluator(old_base, new_base, t, model);
@@ -852,7 +872,7 @@ MATRIX_T* old_get_model_prob_matrix(const EVOMODEL_T* model, double t) {
   return prob_matrix;
 }
 
-BOOLEAN_T uses_halpern_bruno(const EVOMODEL_T* model) {
+bool uses_halpern_bruno(const EVOMODEL_T* model) {
 
   assert(model != NULL);
 
@@ -865,6 +885,7 @@ BOOLEAN_T uses_halpern_bruno(const EVOMODEL_T* model) {
   * page 203.
   ****************************************************************************/
 static double inner_tamura_nei_evaluator(
+    ALPH_T *dna_alph, //TODO this looks like it could benefit from further investigation
     const char old_base, 
     const char new_base, 
     const double t,
@@ -879,8 +900,8 @@ static double inner_tamura_nei_evaluator(
 {
 
   // Check that both bases are in the alphabet.
-  assert(strchr(alphabet, old_base));
-  assert(strchr(alphabet, new_base));
+  assert(alph_is_concrete(dna_alph, old_base));
+  assert(alph_is_concrete(dna_alph, new_base));
 
   // Derive intermediate values used by model.
   const double Pr = Pa + Pg;  // Pyrimidine freq.
@@ -981,6 +1002,7 @@ static double jc_evaluator(
   double non_specific_rate = 4.0 * rate / 3.0;
 
   return inner_tamura_nei_evaluator(
+    model->alph,
     old_base, 
     new_base, 
     t, 
@@ -1029,6 +1051,7 @@ static double k2_evaluator(
     / ( 2.0 * Pr * Py * (1.0 + transition_transversion));
 
   return inner_tamura_nei_evaluator(
+    model->alph,
     old_base,
     new_base,
     t,
@@ -1073,6 +1096,7 @@ static double f81_evaluator(
   double non_specific_rate = rate / (3.0 * Pr * Py);
 
   return inner_tamura_nei_evaluator(
+    model->alph,
     old_base, 
     new_base, 
     t, 
@@ -1126,6 +1150,7 @@ static double f84_evaluator(
     rate / ( 2.0 * Pr * Py * (1.0 + transition_transversion));
 
   return inner_tamura_nei_evaluator(
+    model->alph,
     old_base, 
     new_base, 
     t, 
@@ -1177,6 +1202,7 @@ static double hky_evaluator(
     rate / ( 2.0 * Pr * Py * (1.0 + transition_transversion));
 
   return inner_tamura_nei_evaluator(
+    model->alph,
     old_base, 
     new_base, 
     t, 
@@ -1229,6 +1255,7 @@ static double tamura_nei_evaluator(
     = rate / ( 2.0 * Pr * Py * (1.0 + transition_transversion));
 
   return inner_tamura_nei_evaluator(
+    model->alph,
     new_base, 
     old_base, 
     t,
@@ -1249,6 +1276,7 @@ static double tamura_nei_evaluator(
   * "Inferring Phylogenies", page 201.
   ****************************************************************************/
 static  MATRIX_T* make_rate_matrix(
+  ALPH_T* dna_alph,
   ARRAY_T* equil_freqs,
   const double B,  // non-specific rate
   const double Ar, // purine-purine rate
@@ -1257,10 +1285,11 @@ static  MATRIX_T* make_rate_matrix(
 {
   // Turn parameters into intermediate values
   // used by model
-  double Pa = get_array_item(alphabet_index('A', alphabet), equil_freqs);
-  double Pc = get_array_item(alphabet_index('C', alphabet), equil_freqs);
-  double Pg = get_array_item(alphabet_index('G', alphabet), equil_freqs);
-  double Pt = get_array_item(alphabet_index('T', alphabet), equil_freqs);
+  // TODO investigate further, hardcoding of symbols should be avoided if possible
+  double Pa = get_array_item(alph_index(dna_alph, 'A'), equil_freqs);
+  double Pc = get_array_item(alph_index(dna_alph, 'C'), equil_freqs);
+  double Pg = get_array_item(alph_index(dna_alph, 'G'), equil_freqs);
+  double Pt = get_array_item(alph_index(dna_alph, 'T'), equil_freqs);
   double Pr = Pa + Pg;
   double Py = Pc + Pt;
 
