@@ -23,6 +23,7 @@
 #include "psp.h"
 #include "ushuffle.h"
 #include "string-list.h"
+#include "citation.js.h"
 
 // priors 
 #define PROTEIN_PLIB "prior30.plib"
@@ -304,7 +305,7 @@ void init_meme(
 #endif
 
 #ifndef PARALLEL
-  if (np != NULL) {
+  if (np != NULL && !NO_STATUS) {
     fprintf(stderr, "Option '-p' given but Parallel MEME not configured. Running Serial MEME.\n");
   } 
 #endif
@@ -361,6 +362,11 @@ void init_meme(
     exit(1);
   }
 
+  // set nmotifs to MAX_NMOTIFS if -evt was specified.
+  if (evt != BIG) {
+    nmotifs = MAX_NMOTIFS;
+  }
+
   // get the objective function type
   if (!strcmp(obj, "classic")) {
     objfun = Classic;
@@ -604,6 +610,13 @@ void init_meme(
       fprintf(stderr, "Setting 'searchsize' to the actual size of primary sequences: %d\n", search_size);
     }
   } else {
+    // Don't allow any sequences longer than search_size.
+    if (dataset->max_slength > search_size) {
+      fprintf(stderr, 
+        "Your (primary) dataset must not contain sequences longer than set by -searchsize: %d.\n"
+        "It contains sequence(s) of length %ld.\n", search_size, dataset->max_slength);
+      exit(1);
+    }
     search_size = MIN(dataset->total_res, search_size);
     // max_words scales as sqrt(search_size) so that overall runtime grows as n.
     max_words = MIN(dataset->total_res, 60000 + sqrt(search_size));
@@ -664,7 +677,6 @@ void init_meme(
     fprintf(stderr, "Too few sequences will be searched (%d); increase '-searchsize'.\n", search_size_nseqs);
     exit(1);
   }
-  //if (!NO_STATUS)fprintf(stderr, "EM: search_size %d nseqs = %d search_size_nseqs %d\n", search_size, nseqs, search_size_nseqs);
 
   // Assign the primary and control sequence groups to be used for each task.
   GROUP_T primary_groups = { .em={false,false,false}, .trim={false,false,false}, .pvalue={false,false,false}, .nsites={false,false,false} };
@@ -1296,15 +1308,7 @@ void init_meme(
   char *program = "MEME - Motif discovery tool";
   char *info = "For further information on how to interpret these results please access " SITE_URL ".\n"
     "To get a copy of the MEME Suite software please access " SOURCE_URL ".\n";
-  char *cite = 
-    "If you use this program in your research, please cite:\n"
-    "\n"
-    "Timothy L. Bailey and Charles Elkan,\n"
-    "\"Fitting a mixture model by expectation maximization to discover\n"
-    "motifs in biopolymers\", Proceedings of the Second International\n"
-    "Conference on Intelligent Systems for Molecular Biology, pp. 28-36,\n"
-    "AAAI Press, Menlo Park, California, 1994.\n";
-  banner(program, info, cite, *text_output);
+  banner(program, info, MEME_CITE, *text_output);
 
   // cleanup 
   free(plib_name);
