@@ -159,7 +159,7 @@ static void process_command_line(
      "                                  (default 1.0)\n"
      "  --bfile <file>                File containing n-order Markov background model\n"
      "  --hardmask                    Nucleotides in lower case will be converted to 'N' \n"
-     "                                   preventing them from being considred in motif matches\n"
+     "                                   preventing them from being considered in motif matches\n"
      "  --max-gap <value>             Maximum allowed distance between adjacent hits;\n"
      "                                  (default = 50)\n"
      "  --max-stored-scores <value>   Maximum number of matches that will be stored in memory;\n"
