@@ -772,8 +772,10 @@ void init_meme_background (
     int old_order = order;
     back = load_markov_model(dataset->alph, &order, bfile);
     if (old_order != -1 && old_order != order) {
-      fprintf(stderr, "WARNING: background order %d requested by the given file only contains order %d.\n",
-	order, old_order);
+      fprintf(stderr, 
+        "WARNING: Background order %d requested but the given file only contains order %d.\n"
+        "         Setting -markov_order to %d\n",
+	old_order, order, order);
     }
   } else if (order != -1) {		// -markov_order given
     back = get_markov_from_sequences(seqfile, &order, 0.1, dataset->alph, alph_file, alphabet_type, rc);
