@@ -1,5 +1,5 @@
 /*
- * $Id: message.c 3915 2009-07-21 07:10:21Z tbailey $
+ * $Id$
  * 
  * $Log$
  * Revision 1.1.1.1.2.1  2006/01/31 08:34:53  tbailey
@@ -47,7 +47,7 @@ void balance_loop1(SAMPLE **samples, int n_samples)
   int iseq, inode, seq_start, seq_end;
 
   /* Allocate storage for the sequence parameters. */
-  start_n_end = (SEQ_PARAMS *)mymalloc(sizeof(SEQ_PARAMS));
+  start_n_end = (SEQ_PARAMS *)mm_malloc(sizeof(SEQ_PARAMS));
 
   /* First count how many residues we have. */
   for (iseq = 0, n_residues = 0; iseq < n_samples; iseq++)
@@ -55,6 +55,7 @@ void balance_loop1(SAMPLE **samples, int n_samples)
   
   /* Calculate the number of residues per node. */
   residues_per_node = MAX(1, n_residues / mpNodes());
+  if (!NO_STATUS) fprintf(stderr, "BALANCE: samples %d chars %d nodes %d chars/node %.0f\n", n_samples, n_residues, mpNodes(), residues_per_node);
   
   /*
   fprintf(stderr, "n_residues=%d residues_per_node=%g\n", n_residues,
@@ -265,29 +266,71 @@ void theta_packer(
  *
  * void max_packets
  *
- * Given two packets, select the best logev.
+ * Given two packets, select the one with the best sig.
  * Ties are resolved in favor of:
  *   1) lowest start width
  *   2) lowest start nsites
+ * in Classic mode.  Otherwise, we first break ties using
+ *   1) model width
+ *   2) model LLR
+ * and then the previous two rules are applied.
  * This function is used in the reduction across models.
  *
  **********************************************************************/
-void max_packets (void *f_data, void *f_result, int *f_length,
+void max_packets(void *f_data, void *f_result, int *f_length,
 		  MPI_Datatype *datatype)
 {
-  double rd = ((REDUCE_PACKET *)f_result)->data;
-  double dd = ((REDUCE_PACKET *)f_data)->data;
-  double rw = ((REDUCE_PACKET *)f_result)->s_width;
-  double dw = ((REDUCE_PACKET *)f_data)->s_width;
-  double rn = ((REDUCE_PACKET *)f_result)->s_nsites;
-  double dn = ((REDUCE_PACKET *)f_data)->s_nsites;
+  double r_sig = ((REDUCE_PACKET *)f_result)->sig;
+  double d_sig = ((REDUCE_PACKET *)f_data)->sig;
+  double r_sw = ((REDUCE_PACKET *)f_result)->s_width;
+  double d_sw = ((REDUCE_PACKET *)f_data)->s_width;
+  double r_sn = ((REDUCE_PACKET *)f_result)->s_nsites;
+  double d_sn = ((REDUCE_PACKET *)f_data)->s_nsites;
+  double r_w = ((REDUCE_PACKET *)f_result)->width;
+  double r_nd = ((REDUCE_PACKET *)f_result)->nsites_dis;
+  double d_nd = ((REDUCE_PACKET *)f_data)->nsites_dis;
+  double d_w = ((REDUCE_PACKET *)f_data)->width;
+  double r_llr = ((REDUCE_PACKET *)f_result)->llr;
+  double d_llr = ((REDUCE_PACKET *)f_data)->llr;
+  bool classic = ((REDUCE_PACKET *)f_result)->classic == 1;
 
   /* Compare the data in the two packets. */
-  if ( dd < rd || (dd == rd && dw < rw) || (dd == rd && dw == rw && dn < rn) ) {
+  // These rules must agree with discretize() and
+  // the logic of save_candidate() or serial and
+  // parallel results can differ.
+  bool data_wins;
+  if (
+    (d_sig < r_sig)
+    || (!classic 			// not Classic
+      && (				// and data better
+	(d_sig == r_sig && d_w < r_w) 
+	|| (d_sig == r_sig && d_w == r_w && d_llr > r_llr)
+	|| (d_sig == r_sig && d_w == r_w && d_llr == r_llr && d_sw < r_sw)
+	|| (d_sig == r_sig && d_w == r_w && d_llr == r_llr && d_sw == r_sw && d_sn < r_sn)
+      )
+    )
+    || (classic				// Classic
+      && (				// and data better
+	(d_sig == r_sig && d_sw < r_sw) 
+	|| (d_sig == r_sig && d_sw == r_sw && d_sn < r_sn )
+      )
+    )
+  ) {
+    data_wins = true;
+  } else {
+    data_wins = false;
+  }
+
+  // Now do the reduction if data wins.
+  if (data_wins) {
     /* Put the data into the result position. */
-    ((REDUCE_PACKET *)f_result)->data = dd;
-    ((REDUCE_PACKET *)f_result)->s_width = dw;
-    ((REDUCE_PACKET *)f_result)->s_nsites = dn;
+    ((REDUCE_PACKET *)f_result)->sig = d_sig;
+    ((REDUCE_PACKET *)f_result)->s_width = d_sw;
+    ((REDUCE_PACKET *)f_result)->s_nsites = d_sn;
+    ((REDUCE_PACKET *)f_result)->width = d_w;
+    ((REDUCE_PACKET *)f_result)->nsites_dis = d_nd;
+    ((REDUCE_PACKET *)f_result)->llr = d_llr;
+    ((REDUCE_PACKET *)f_result)->classic = ((REDUCE_PACKET *)f_data)->classic;
     ((REDUCE_PACKET *)f_result)->ID = ((REDUCE_PACKET *)f_data)->ID;
   }
 }
@@ -320,20 +363,28 @@ void reduce_across_models(
   mpReduceAdd(&(model->iter));
 
   /* Package the best model's sig together with the processor ID. */
-  a_packet.data = model->logev;
+  // With the Classic objective function, ties are broken in
+  // by selecting the model that would be evaluated *first* in serial mode.
+  // For other objective functions, model width and LLR are used
+  // before that to break ties.
+  a_packet.sig = model->logev;
   a_packet.s_width = model->pw;
   a_packet.s_nsites = model->psites;
+  a_packet.width = (double)model->w;
+  a_packet.nsites_dis = (double)model->nsites_dis;
+  a_packet.llr = model->llr;
+  a_packet.classic = model->objfun==Classic ? 1.0 : 0.0;
   a_packet.ID = (double)mpMyID();
     
-  /*fprintf(stdout, "%d: Prior score=%g\n", mpMyID(), a_packet.data);*/
+  /*fprintf(stdout, "%d: Prior score=%g\n", mpMyID(), a_packet.sig);*/
 
   /* Reduce across all the sig-ID pairs. */
   /* Make sure we have a handle for the reduction function. */
   if (init == 0) {
     init = 1;
-    MPI_Type_contiguous(4, MPI_DOUBLE, &reduction_packet_type);
+    MPI_Type_contiguous(8, MPI_DOUBLE, &reduction_packet_type);
     MPI_Type_commit(&reduction_packet_type);
-    MPI_Op_create(max_packets, TRUE, &max_packets_op);
+    MPI_Op_create(max_packets, true, &max_packets_op);
   }
 
   /* Do the reduction. */
@@ -341,7 +392,7 @@ void reduce_across_models(
 		reduction_packet_type, max_packets_op, MPI_COMM_WORLD);
 
   /*fprintf(stdout, "%d: Winner=%d score=%g\n", mpMyID(), (int)best_packet.ID,
-	  best_packet.data);*/
+	  best_packet.sig);*/
 
   /* The losers store their theta matrix and maxima pointers so they don't get
      overwritten in the upcoming broadcast. */
@@ -397,28 +448,32 @@ void max_s_packets(void *f_data, void *f_result, int *f_length,
 
     /* Get the two scores. */
     rs = ((S_POINT_PACKET *)f_result + i_nsites0)->score;
-    ri = ((S_POINT_PACKET *)f_result + i_nsites0)->iseq;
-    rj = ((S_POINT_PACKET *)f_result + i_nsites0)->ioff;
     ds = ((S_POINT_PACKET *)f_data + i_nsites0)->score;
+    ri = ((S_POINT_PACKET *)f_result + i_nsites0)->iseq;
     di = ((S_POINT_PACKET *)f_data + i_nsites0)->iseq;
+    rj = ((S_POINT_PACKET *)f_result + i_nsites0)->ioff;
     dj = ((S_POINT_PACKET *)f_data + i_nsites0)->ioff;
 
-    /* Compare the two scores. */
+    /* Compare the two scores breaking ties so that starting points
+       tested first in serial mode win. */
+    int data_wins = false;
     if (ds > rs || (ds == rs && di < ri) || (ds == rs && di == ri && dj < rj)) {
       /* Copy the new data into the result location. */
       ((S_POINT_PACKET *)f_result + i_nsites0)->score = ds;
       ((S_POINT_PACKET *)f_result + i_nsites0)->iseq = di;
       ((S_POINT_PACKET *)f_result + i_nsites0)->ioff = dj;
+      data_wins = true;
     }
+
   }
-}
+} // max_s_packets
 
 /**********************************************************************
  *
  * void reduce_across_s_points
  *
  * Do a reduction across an array of starting points. For each
- * position in the array, the starting point with the lowest value in
+ * position in the array, the starting point with the largest value in
  * the 'score' field gets propagated to every node.
  *
  * This function is called by subseq7.c.
@@ -466,7 +521,7 @@ void reduce_across_s_points(
     init = 1;
     MPI_Type_contiguous(3, MPI_DOUBLE, &s_point_packet_type);
     MPI_Type_commit(&s_point_packet_type);
-    MPI_Op_create(max_s_packets, TRUE, &max_s_packets_op);
+    MPI_Op_create(max_s_packets, true, &max_s_packets_op);
   }
 
   /* Do the reduction. */
@@ -505,7 +560,7 @@ void reduce_across_s_points(
 	Get the precalculated astarting and ending sequences and offsets.
 */
 /**********************************************************************/
-extern void get_start_n_end (
+void get_start_n_end (
   int *start_seq,			/* starting sequence number */
   int *start_off,			/* offset in starting sequence */
   int *end_seq,				/* ending sequence number */
@@ -524,7 +579,7 @@ extern void get_start_n_end (
  * Print some of the contents of a model.
  * (Debugging only.)
  **********************************************************************/
-extern void print_model(
+void print_model(
    char *label,
    MODEL *model
 )
