@@ -6,19 +6,20 @@
  * COPYRIGHT: 2004, UW
  ********************************************************************/
 
+#define DEFINE_GLOBALS
 #include <assert.h>
 #include <stdio.h>
 #include <string.h>
+#include <time.h>
 #include "alignment.h"
 #include "alphabet.h"
 #include "cisml.h"
 #include "dir.h"
 #include "evomodel.h"
 #include "fasta-io.h"
-#include "hash_alph.h"
-#include "meme-io.h"
 #include "mhmm-state.h"
 #include "motif.h"
+#include "motif-in.h"
 #include "motiph-scoring.h"
 #include "pssm.h"
 #include "seq.h"
@@ -29,23 +30,22 @@
 char* program_name = NULL;
 VERBOSE_T verbosity = NORMAL_VERBOSE;
 
-// Nucleotide alphabet order as in motif.h
-extern char alphabet[];
-
 /*******************************************************************
   Print the column frequency distribution.
  ********************************************************************/
 static void print_col_frequencies(
+  ALPH_T *alph,
   ARRAY_T* alignment_column_freqs
 )
 {
   int i;
   int num_freqs = get_array_length(alignment_column_freqs);
-  int alph_size = get_alph_size(ALPH_SIZE);
-  int num_leaves = NINT(log(num_freqs)/log(alph_size));
+  int asize = alph_size_core(alph);
+  int num_leaves = NINT(log(num_freqs)/log(asize));
   char* alignment_col = mm_malloc((num_leaves + 1) * sizeof(char));
   for (i=0; i<num_freqs; i++) {
     unhash_alignment_col(
+      alph,
       i,                              //col_index
       alignment_col,
       num_leaves
@@ -60,24 +60,25 @@ static void print_col_frequencies(
  *************************************************************************/
 int main(int argc, char *argv[]) {
 
-  BOOLEAN_T allow_clobber = TRUE;
-  BOOLEAN_T compute_qvalues = TRUE;
-  BOOLEAN_T print_col_freqs = FALSE;
-  BOOLEAN_T print_trimmed_tree = FALSE;
-  BOOLEAN_T scan_both_strands = TRUE;
-  BOOLEAN_T bls_both_strands = TRUE;
-  BOOLEAN_T output_qthresh_set = FALSE;
-  BOOLEAN_T text_only = FALSE;
-  BOOLEAN_T use_file_list = FALSE;
-  BOOLEAN_T use_halpern_bruno = FALSE;
-  BOOLEAN_T use_BLS = FALSE;
+  bool allow_clobber = true;
+  bool compute_qvalues = true;
+  bool print_col_freqs = false;
+  bool print_trimmed_tree = false;
+  bool scan_both_strands = true;
+  bool bls_both_strands = true;
+  bool output_qthresh_set = false;
+  bool text_only = false;
+  bool use_file_list = false;
+  bool use_halpern_bruno = false;
+  bool use_BLS = false;
   int bls_distance = 20;
   int ref_seq_index = 0;  // Which sequence provides reference coords.
   int max_stored_scores = 100000;
   //FIXME: Eventually, above should be accessible via a command-line switch.
+  char* command_line = NULL;
   char* bg_filename = NULL;
   char *output_dirname = "motiph_out";
-  char* ustar_label = NULL;	// TLB; create uniform star tree
+  char* ustar_label = NULL;     // TLB; create uniform star tree
   double bg_rate = 1.0;
   double fg_rate = 1.0;
   double gap_cost = 0.0;
@@ -91,6 +92,7 @@ int main(int argc, char *argv[]) {
   COLUMN_FREQS_TYPE_T column_freqs_type = SIMULATED_COL_FREQS;
   STRING_LIST_T* selected_motifs = NULL;
   OBJECT_LIST_T* alignment_col_freqs_list = NULL;
+  bool report_err = false;
 
   program_name = "motiph";
 
@@ -101,9 +103,10 @@ int main(int argc, char *argv[]) {
   // Define command line options.
   // FIXME: Note that if you add or remove options you
   // must change n_options.
-  int n_options = 27;
+  int n_options = 29;
   cmdoption const motiph_options[] = {
     {"bg", REQUIRED_VALUE},
+    {"bfile", REQUIRED_VALUE},
     {"bgfile", REQUIRED_VALUE},
     {"bls-dist", REQUIRED_VALUE},
     {"column-freqs", REQUIRED_VALUE},
@@ -126,6 +129,7 @@ int main(int argc, char *argv[]) {
     {"print-trimmed-tree", NO_VALUE},
     {"pseudocount", REQUIRED_VALUE},
     {"pur-pyr", REQUIRED_VALUE},
+    {"seed", REQUIRED_VALUE},
     {"text", NO_VALUE},
     {"transition-transversion", REQUIRED_VALUE},
     {"ustar", REQUIRED_VALUE},
@@ -136,8 +140,8 @@ int main(int argc, char *argv[]) {
 
   // Define the usage message.
   char *usage = 
-   "USAGE: motiph [options] <alignment> <tree> <motif>\n"
-   "\n"
+  "Usage: motiph [options] <alignment> <tree> <motif>\n"
+  "\n"
   "   Options:\n"
 
   // Evolutionary model parameters.
@@ -152,13 +156,13 @@ int main(int argc, char *argv[]) {
   " (default=f81)\n"
   "     --pur-pyr <float> (default=1.0)\n"
   "     --transition-transversion <float> (default=0.5)\n"
-  "     --ustar <label>\n"	// TLB; create uniform star tree
+  "     --ustar <label>\n"      // TLB; create uniform star tree
 
   // Motif parameters.
   "     --motif <id> (default=all)\n"
 
   // Miscellaneous parameters
-  "     --bgfile <background> (default from motif file)\n"
+  "     --bfile <background> (default from motif file)\n"
   "     --list\n"
   "     --no-qvalue (default false)\n"
   "     --norc (default false)\n"
@@ -172,6 +176,7 @@ int main(int argc, char *argv[]) {
   "     --pseudocount <float> (default=0.1)\n"
   "     --output-pthresh <float> (default 1e-4)\n"
   "     --output-qthresh <float> (default 1.0)\n"
+  "     --seed <long> (default from time())\n"
   "     --text (default false)\n"
   "     --verbosity [1|2|3|4] (default 2)\n"
   "\n";
@@ -182,7 +187,10 @@ int main(int argc, char *argv[]) {
     die("Error processing command line options: option name too long.\n");
   }
 
-  while (TRUE) {
+  // Default seed the random number generator.
+  long seed = time(0);
+
+  while (true) {
     int c = 0;
     char* option_name = NULL;
     char* option_value = NULL;
@@ -200,14 +208,14 @@ int main(int argc, char *argv[]) {
     if (strcmp(option_name, "bg") == 0){
       bg_rate = atof(option_value);
     }
-    else if (strcmp(option_name, "bgfile") == 0){
+    else if (strcmp(option_name, "bfile") == 0 || strcmp(option_name, "bgfile") == 0){
       bg_filename = option_value;
     }     
     else if (strcmp(option_name, "bls-dist") == 0){
       bls_distance = atoi(option_value);
     }   
     //else if (strcmp(option_name, "bls") == 0){
-    //  use_BLS = TRUE;
+    //  use_BLS = true;
     //} 
     else if (strcmp(option_name, "column-freqs") == 0) {
       if (strcmp(option_value, "simulated") == 0) {
@@ -223,9 +231,9 @@ int main(int argc, char *argv[]) {
     }
     else if (strcmp(option_name, "flip") == 0) {
       if (strcmp(option_value, "false") == 0) {
-        bls_both_strands = FALSE;
+        bls_both_strands = false;
       } else {
-        bls_both_strands = TRUE;
+        bls_both_strands = true;
       }
     } 
     else if (strcmp(option_name, "gap") == 0){
@@ -245,10 +253,10 @@ int main(int argc, char *argv[]) {
       gap_cost = atof(option_value);
     }
     else if (strcmp(option_name, "hb") == 0){
-        use_halpern_bruno = TRUE;
+        use_halpern_bruno = true;
     }
     else if (strcmp(option_name, "list") == 0){
-      use_file_list = TRUE;
+      use_file_list = true;
     }
     else if (strcmp(option_name, "model") == 0) {
       if (strcmp(option_value, "jc") == 0) {
@@ -268,7 +276,7 @@ int main(int argc, char *argv[]) {
 
       } else if (strcmp(option_value, "bls") == 0) {
         model_type = SINGLE_MODEL;
-	use_BLS = TRUE;
+        use_BLS = true;
 
       } else if (strcmp(option_value, "average") == 0) {
         model_type = AVERAGE_MODEL;
@@ -281,26 +289,26 @@ int main(int argc, char *argv[]) {
       max_stored_scores = (int)atof(option_value);
     }
     else if (strcmp(option_name, "motif") == 0){
-        if (selected_motifs == NULL) {
-          selected_motifs = new_string_list();
-        }
-       add_string(option_value, selected_motifs);
+      if (selected_motifs == NULL) {
+        selected_motifs = new_string_list();
+      }
+      add_string(option_value, selected_motifs);
     }
     else if (strcmp(option_name, "no-qvalue") == 0) {
-      compute_qvalues = FALSE;
+      compute_qvalues = false;
     }
     else if (strcmp(option_name, "norc") == 0) {
-      scan_both_strands = FALSE;
+      scan_both_strands = false;
     }
     else if (strcmp(option_name, "o") == 0){
       // Set output directory with no clobber
       output_dirname = option_value;
-      allow_clobber = FALSE;
+      allow_clobber = false;
     }
     else if (strcmp(option_name, "oc") == 0){
       // Set output directory with clobber
       output_dirname = option_value;
-      allow_clobber = TRUE;
+      allow_clobber = true;
     }
     else if (strcmp(option_name, "output-pthresh") == 0){
       output_pthresh = atof(option_value);
@@ -309,43 +317,52 @@ int main(int argc, char *argv[]) {
     else if (strcmp(option_name, "output-qthresh") == 0){
       output_qthresh = atof(option_value);
       output_pthresh = 1.0;
-      output_qthresh_set = TRUE;
+      output_qthresh_set = true;
     }
     else if (strcmp(option_name, "print-col-freqs") == 0){
-      print_col_freqs = TRUE;
+      print_col_freqs = true;
     }
 
     else if (strcmp(option_name, "print-trimmed-tree") == 0){
-      print_trimmed_tree = TRUE;
+      print_trimmed_tree = true;
     }
     else if (strcmp(option_name, "pseudocount") == 0){
       pseudocount = atof(option_value);
     }
     else if (strcmp(option_name, "pur-pyr") == 0){
-        purine_pyrimidine = atof(option_value);
+      purine_pyrimidine = atof(option_value);
+    }
+    else if (strcmp(option_name, "seed") == 0){
+      seed = atol(option_value);
     }
     else if (strcmp(option_name, "text") == 0){
-      text_only = TRUE;
+      text_only = true;
     }
     else if (strcmp(option_name, "transition-transversion") == 0){
-        transition_transversion = atof(option_value);
+      transition_transversion = atof(option_value);
     }
-    else if (strcmp(option_name, "ustar") == 0){	// TLB; create uniform star tree
-        ustar_label = option_value;
+    else if (strcmp(option_name, "ustar") == 0){        // TLB; create uniform star tree
+      ustar_label = option_value;
     }
     else if (strcmp(option_name, "verbosity") == 0){
       verbosity = atoi(option_value);
     }
   }
+ 
+  // Record the command line
+  command_line = get_command_line(argc, argv);
+
+  // Initialize random number generator
+  srand_mt(seed);
 
   // Check that the qvalue options are consistent
-  if (compute_qvalues == FALSE && output_qthresh_set == TRUE) {
+  if (compute_qvalues == false && output_qthresh_set == true) {
     die("The --no-qvalue option cannot be used with the --output-qthresh options");
   }
 
   // Must have tree and alignment file names
   if (argc != option_index + 3) {
-    fprintf(stderr, usage);
+    fprintf(stderr, "%s", usage);
     exit(EXIT_FAILURE);
   }
 
@@ -383,41 +400,33 @@ int main(int argc, char *argv[]) {
    * Read the motifs.
    **********************************************/
   char* meme_filename = argv[option_index];
-  int num_motifs = 0;
-  MOTIF_T motifs[2 * MAX_MOTIFS];
-  STRING_LIST_T* motif_occurrences = NULL;
-  BOOLEAN_T has_reverse_strand = FALSE;
+  ARRAYLST_T *motifs;
   ARRAY_T* bg_freqs = NULL;
   option_index++;
 
-  read_meme_file(
-		 meme_filename,
-		 bg_filename,
-		 pseudocount,
-		 &num_motifs,
-		 motifs,
-		 &motif_occurrences,
-		 &has_reverse_strand,
-		 &bg_freqs
-		 );
+  MREAD_T *mread;
+  ALPH_T *alph;
 
-  // We don't make use of the motif occurence data
-  free_string_list(motif_occurrences);
+  // create motif parser
+  mread = mread_create(meme_filename, OPEN_MFILE, scan_both_strands);
+  mread_set_bg_source(mread, bg_filename, NULL);
+  mread_set_pseudocount(mread, pseudocount);
+  // read motifs
+  motifs = mread_load(mread, NULL);
+  alph = alph_hold(mread_get_alphabet(mread));
+  bg_freqs = mread_get_background(mread);
+  // destroy motif parser
+  mread_destroy(mread);
+  // check motifs
+  if (arraylst_size(motifs) == 0) die("No motifs in motif file.");
+  // TODO FIXME check that this is really required
+  if (!alph_is_builtin_dna(alph)) die("Expected DNA motifs");
+  assert(bg_freqs != NULL);
 
-  if (scan_both_strands == TRUE ) {
-    // Set up hash tables for computing reverse complement
-    setup_hash_alph(DNAB);
-    setalph(0);
-    // Correct background by averaging on freq. for both strands.
-    average_freq_with_complement(bg_freqs);
-    int alph_size = get_alph_size(ALPH_SIZE);
-    normalize_subarray(0, alph_size, 0.0, bg_freqs);
-    fill_in_ambiguous_chars(FALSE, bg_freqs);
+  if (scan_both_strands == true ) {
+    calc_ambigs(alph, false, bg_freqs);
     // Make reverse complement motifs.
-    int num_motifs_after_rc = num_motifs;
-    add_reverse_complements(&num_motifs_after_rc, motifs);
-    assert(num_motifs_after_rc == (2 * num_motifs));
-    num_motifs = num_motifs_after_rc;
+    add_reverse_complements(motifs);
   }
 
   // OK If we use the BLS Scan method we need to set up the inverse
@@ -444,11 +453,11 @@ int main(int argc, char *argv[]) {
   if (column_freqs_type == EMPIRICAL_COL_FREQS) {
     if (verbosity >= NORMAL_VERBOSE) {
       fprintf(
-       stderr,
-       "Computing column frequencies from multiple alignment file(s).\n"
+        stderr,
+        "Computing column frequencies from multiple alignment file(s).\n"
       );
     }
-    alignment_col_freqs_list = get_alignment_column_freqs_list(filenames, TRUE);
+    alignment_col_freqs_list = get_alignment_column_freqs_list(alph, filenames, true);
   } else if (column_freqs_type == PRECOMPUTED_COL_FREQS) {
     // FIXME: add switch --column-freqs-file <file> and read that
     // file of column frequency distributions for different sets of species.
@@ -456,13 +465,13 @@ int main(int argc, char *argv[]) {
   } else {
     // Using SIMULATED column frequencies.  Initialize list.
     alignment_col_freqs_list = new_object_list(equal_string_lists,
-					       (void*)copy_string_list,
-					       free_string_list,
-					       free_array);
+        (void*)copy_string_list,
+        free_string_list,
+        free_array);
   } // column freqs list
 
   // Create cisml data structure for recording results
-  CISML_T *cisml = allocate_cisml("motiph", meme_filename, "clustal-w alignment");
+  CISML_T *cisml = allocate_cisml("motiph", command_line, meme_filename, "clustal-w alignment");
   set_cisml_site_pvalue_cutoff(cisml, output_pthresh);
   set_cisml_site_qvalue_cutoff(cisml, output_qthresh);
 
@@ -471,10 +480,10 @@ int main(int argc, char *argv[]) {
   **************************************************************/
   int n_scored = 0;    // number of positions that were scored
   int motif_index;
-  for (motif_index = 0; motif_index < num_motifs; motif_index++) {
+  for (motif_index = 0; motif_index < arraylst_size(motifs); motif_index++) {
 
-    MOTIF_T* motif = &(motifs[motif_index]);
-    char* motif_id = get_motif_id(motif);
+    MOTIF_T* motif = (MOTIF_T*)arraylst_get(motif_index, motifs);
+    char* motif_id = (scan_both_strands ? get_motif_st_id(motif) : get_motif_id(motif));
     char* bare_motif_id = motif_id;
 
     // Create cisml pattern and add to cisml record
@@ -489,17 +498,17 @@ int main(int argc, char *argv[]) {
         // The selected  motif id won't included a strand indicator.
         bare_motif_id++;
       }
-      if (have_string(bare_motif_id, selected_motifs) == FALSE) {
+      if (have_string(bare_motif_id, selected_motifs) == false) {
         continue;
       }
     }
 
-    int window_size = motif->length;
+    int window_size = get_motif_length(motif);
     if (verbosity >= NORMAL_VERBOSE) {
       fprintf(
         stderr,
         "Using motif %s of width %d.\n",
-        motif_id, motif->length
+        motif_id, get_motif_length(motif)
       );
     }
 
@@ -518,9 +527,9 @@ int main(int argc, char *argv[]) {
     PSSM_T* pssm = NULL;
     ARRAY_T* alignment_col_freqs = NULL;
     OBJECT_LIST_T* pssm_list = new_object_list(equal_string_lists,
-					       (void*)copy_string_list,
-					       free_string_list,
-					       free_pssm);
+        (void*)copy_string_list,
+        free_string_list,
+        free_pssm);
 
     SCORED_SITES_T* site_list = new_scored_sites();
 
@@ -536,28 +545,28 @@ int main(int argc, char *argv[]) {
     OBJECT_LIST_T* inverse_pssm_list = NULL;
 
     if( use_BLS && bls_both_strands ) {
-	// if the motif index is even then the inverse comes after,
-	// otherwise it is before it.
-	int inverse_index = motif_index + 1;
-	if( (motif_index % 2) > 0)
-		inverse_index = motif_index - 1;
+        // if the motif index is even then the inverse comes after,
+        // otherwise it is before it.
+        int inverse_index = motif_index + 1;
+        if( (motif_index % 2) > 0)
+          inverse_index = motif_index - 1;
 
-	inverse_motif = &(motifs[inverse_index]);
-	inverse_models = make_motif_models(
-      		inverse_motif, 
-      		bg_freqs,
-      		model_type,
-      		fg_rate, 
-      		bg_rate, 
-      		purine_pyrimidine, 
-      		transition_transversion, 
-      		use_halpern_bruno
-    	);
+        inverse_motif = (MOTIF_T*)arraylst_get(inverse_index, motifs);
+        inverse_models = make_motif_models(
+            inverse_motif, 
+            bg_freqs,
+            model_type,
+            fg_rate, 
+            bg_rate, 
+            purine_pyrimidine, 
+            transition_transversion, 
+            use_halpern_bruno
+        );
 
-    	inverse_pssm_list = new_object_list(equal_string_lists,
-					       (void*)copy_string_list,
-					       free_string_list,
-					       free_pssm);
+        inverse_pssm_list = new_object_list(equal_string_lists,
+            (void*)copy_string_list,
+            free_string_list,
+            free_pssm);
     }
 
     // Consider each alignment in turn.
@@ -570,11 +579,11 @@ int main(int argc, char *argv[]) {
       // species alphabetically.
       char* filename = get_nth_string(file_index, filenames);
       ALIGNMENT_T* alignment = read_alignment_from_file(filename,
-							TRUE, // Sort?
-							TRUE, // Remove gaps?
-							&current_ref_seq_index);
+          true, // Sort?
+          true, // Remove gaps?
+          &current_ref_seq_index);
       if (verbosity >= NORMAL_VERBOSE) {
-	fprintf(stderr, "Read alignment from %s.\n", filename);
+        fprintf(stderr, "Read alignment from %s.\n", filename);
       }
 
       // Make sure there are no duplicates.
@@ -589,7 +598,7 @@ int main(int argc, char *argv[]) {
       set_scanned_sequence_length(scanned_seq, get_alignment_length(alignment));
 
       // Trim the tree, eliminating species not in this alignment.
-      TREE_T* trimmed_tree = trim_tree(TRUE, tree, alignment_species);
+      TREE_T* trimmed_tree = trim_tree(true, tree, alignment_species);
 
       // Check that at least one species was found.
       if (trimmed_tree == NULL) {
@@ -606,7 +615,7 @@ int main(int argc, char *argv[]) {
         write_tree(trimmed_tree, stderr);
       }
       STRING_LIST_T* tree_species = make_leaf_list(trimmed_tree);
-      sort_string_list(tree_species); 	// keep species alphabetical
+      sort_string_list(tree_species);   // keep species alphabetical
 
       // Trim the alignment, eliminating species not in this tree.
       ALIGNMENT_T* trimmed_alignment = remove_alignment_seqs(tree_species, alignment);
@@ -614,35 +623,34 @@ int main(int argc, char *argv[]) {
       free_string_list(tree_species);
       free_string_list(alignment_species);
       alignment_species = (get_species_names(trimmed_alignment));
-      sort_string_list(alignment_species);	// sort species alphabetically
-	
+      sort_string_list(alignment_species);      // sort species alphabetically
+        
       // Try to retrieve the pre-computed PSSM from memory.
       pssm = (PSSM_T*)retrieve_object(alignment_species, pssm_list);
       if (use_BLS && bls_both_strands) {
-      	// Try to retrieve the pre-computed inverse PSSM from memory.
-      	inverse_pssm = (PSSM_T*)retrieve_object(alignment_species, inverse_pssm_list);
+        // Try to retrieve the pre-computed inverse PSSM from memory.
+        inverse_pssm = (PSSM_T*)retrieve_object(alignment_species, inverse_pssm_list);
       }
 
 
       // JH - I am going to check whether this needs to be done now
       // so it gets done just the once.
       if (pssm == NULL || (use_BLS && bls_both_strands && inverse_pssm == NULL)) {
-	// TLB; Convert the tree to a uniform star tree with
-	// the target sequence at its center.
-	if (ustar_label != NULL) {
-	  trimmed_tree = 
-		  convert_to_uniform_star_tree(trimmed_tree, ustar_label);
-	  if (trimmed_tree == NULL) 
-		  die("Tree or alignment missing target %s\n", ustar_label);
-	  if (verbosity >= NORMAL_VERBOSE) {
-		  fprintf(
-			  stderr,
-			  "Target %s placed at center of uniform star tree:\n", 
-			  ustar_label
-		  );
-		  write_tree(trimmed_tree, stderr);
-	  }
-	}
+        // TLB; Convert the tree to a uniform star tree with
+        // the target sequence at its center.
+        if (ustar_label != NULL) {
+          trimmed_tree = convert_to_uniform_star_tree(trimmed_tree, ustar_label);
+          if (trimmed_tree == NULL) 
+            die("Tree or alignment missing target %s\n", ustar_label);
+          if (verbosity >= NORMAL_VERBOSE) {
+            fprintf(
+                stderr,
+                "Target %s placed at center of uniform star tree:\n", 
+                ustar_label
+            );
+            write_tree(trimmed_tree, stderr);
+          }
+        }
       }
 
       // Did we fail to retrieve the PSSM?
@@ -654,6 +662,7 @@ int main(int argc, char *argv[]) {
 
         // Build PSSM *matrix* for motiph
         MATRIX_T* pssm_matrix = build_alignment_pssm_matrix(
+          alph,
           alignment_species,
           window_size + 1,
           models,
@@ -678,15 +687,15 @@ int main(int argc, char *argv[]) {
           store_object(
             (void*)alignment_col_freqs,
             (void*)alignment_species,
-	    0, // Score
+            0, // Score
             alignment_col_freqs_list
           );
           if (print_col_freqs && column_freqs_type == SIMULATED_COL_FREQS) {
-            print_col_frequencies(alignment_col_freqs);
+            print_col_frequencies(alph, alignment_col_freqs);
             exit(1);
           }
         } else if (print_col_freqs) {
-          print_col_frequencies(alignment_col_freqs);
+          print_col_frequencies(alph, alignment_col_freqs);
           exit(1);
         }
 
@@ -694,12 +703,15 @@ int main(int argc, char *argv[]) {
         remove_matrix_row(0, pssm_matrix);
 
         // Create a PSSM object. Build tables to translate log-odds scores to p-values.
-	pssm = build_matrix_pssm(
-	  pssm_matrix, 
-	  alignment_col_freqs,
-	  PSSM_RANGE 
-	);
-	free_matrix(pssm_matrix);
+        pssm = build_matrix_pssm(
+          alph,
+          pssm_matrix, 
+          alignment_col_freqs,
+          NULL, // No prior dist
+          1.0, // PSP paramater alpha not used
+          PSSM_RANGE 
+        );
+        free_matrix(pssm_matrix);
 
         // Store them for later use.
         store_object((void*)pssm, (void*)alignment_species, 0, pssm_list);
@@ -723,82 +735,85 @@ int main(int argc, char *argv[]) {
       // doing BLS on both strands
       /* ----------------------------------------------------------------*/
       if (use_BLS && bls_both_strands) {
-      	if (inverse_pssm == NULL) {
-		if (verbosity > NORMAL_VERBOSE) {
-	          	fprintf(stderr, "Building Inverse PSSM for %s.\n",
-	          	combine_string_list(alignment_species, "-"));
-        	}
+        if (inverse_pssm == NULL) {
+          if (verbosity > NORMAL_VERBOSE) {
+            fprintf(stderr, "Building Inverse PSSM for %s.\n",
+            combine_string_list(alignment_species, "-"));
+          }
 
-        	// Build PSSM for Inverse Motif
-                MATRIX_T* inverse_pssm_matrix = build_alignment_pssm_matrix(
-          		alignment_species,
-          		window_size + 1, 
-          		inverse_models, 
-          		trimmed_tree, 
-          		gap_support
-        	);
+          // Build PSSM for Inverse Motif
+          MATRIX_T* inverse_pssm_matrix = build_alignment_pssm_matrix(
+            alph,
+            alignment_species,
+            window_size + 1, 
+            inverse_models, 
+            trimmed_tree, 
+            gap_support
+          );
 
-        	// Get the column frequency distribution for computing p-values.
-        	inverse_alignment_col_freqs = 
-          		retrieve_object(alignment_species, alignment_col_freqs_list);
-        	// Column frequencies not found?
-        	if (alignment_col_freqs == NULL) {
-          		if (column_freqs_type != SIMULATED_COL_FREQS) {
-            			die("No column frequency distribution was given for species: %s\n",
-              			combine_string_list(alignment_species, "-"));
-          		}
-          		// Get the simulated column frequencies from first row of PSSM.
-          		// They are based on the single-letter frequencies and the evo model.
-          		inverse_alignment_col_freqs = allocate_array(get_num_cols(inverse_pssm_matrix)); 
-          		copy_array(get_matrix_row(0, inverse_pssm_matrix), inverse_alignment_col_freqs);
-          		// Save the column frequencies for this species list for future use.
-          		store_object(
-            			(void*)inverse_alignment_col_freqs,
-            			(void*)alignment_species, 
-	    			0, // Score
-            			alignment_col_freqs_list
-          		);
-          		if (print_col_freqs && column_freqs_type == SIMULATED_COL_FREQS) {
-            			print_col_frequencies(inverse_alignment_col_freqs);
-            			exit(1);
-          		}
-        	} else if (print_col_freqs) {
-          		print_col_frequencies(inverse_alignment_col_freqs);
-          		exit(1);
-        	}
+          // Get the column frequency distribution for computing p-values.
+          inverse_alignment_col_freqs = 
+            retrieve_object(alignment_species, alignment_col_freqs_list);
+          // Column frequencies not found?
+          if (alignment_col_freqs == NULL) {
+            if (column_freqs_type != SIMULATED_COL_FREQS) {
+              die("No column frequency distribution was given for species: %s\n",
+              combine_string_list(alignment_species, "-"));
+            }
+            // Get the simulated column frequencies from first row of PSSM.
+            // They are based on the single-letter frequencies and the evo model.
+            inverse_alignment_col_freqs = allocate_array(get_num_cols(inverse_pssm_matrix)); 
+            copy_array(get_matrix_row(0, inverse_pssm_matrix), inverse_alignment_col_freqs);
+            // Save the column frequencies for this species list for future use.
+            store_object(
+              (void*)inverse_alignment_col_freqs,
+              (void*)alignment_species, 
+              0, // Score
+              alignment_col_freqs_list
+            );
+            if (print_col_freqs && column_freqs_type == SIMULATED_COL_FREQS) {
+              print_col_frequencies(alph, inverse_alignment_col_freqs);
+              exit(1);
+            }
+          } else if (print_col_freqs) {
+            print_col_frequencies(alph, inverse_alignment_col_freqs);
+            exit(1);
+          }
 
-        	// Remove first row from the PSSM.  It was saved above if needed.
-        	remove_matrix_row(0, inverse_pssm_matrix);
+          // Remove first row from the PSSM.  It was saved above if needed.
+          remove_matrix_row(0, inverse_pssm_matrix);
 
-        	// Build tables to translate log-odds scores to p-values
-                inverse_pssm = build_matrix_pssm(
-                  inverse_pssm_matrix, 
-                  inverse_alignment_col_freqs,
-		  PSSM_RANGE 
-                );
-		free_matrix(inverse_pssm_matrix);
+          // Build tables to translate log-odds scores to p-values
+          inverse_pssm = build_matrix_pssm(
+            alph,
+            inverse_pssm_matrix, 
+            inverse_alignment_col_freqs,
+            NULL, // No priors
+            1.0, // PSP parameter alpha not used
+            PSSM_RANGE 
+          );
+          free_matrix(inverse_pssm_matrix);
 
-        	// Store them for later use.
-        	store_object((void*)inverse_pssm, (void*)alignment_species, 0, inverse_pssm_list);
+          // Store them for later use.
+          store_object((void*)inverse_pssm, (void*)alignment_species, 0, inverse_pssm_list);
 
-      	} else {
+        } else {
+          // We got the PSSM, so get the other stuff that goes with it.
 
-      		// We got the PSSM, so get the other stuff that goes with it.
+          if (verbosity > NORMAL_VERBOSE) {
+            fprintf(stderr, "Retrieved PSSM for %s.\n",
+            combine_string_list(alignment_species, "-"));
+          }
 
-       		if (verbosity > NORMAL_VERBOSE) {
-          		fprintf(stderr, "Retrieved PSSM for %s.\n",
-          		combine_string_list(alignment_species, "-"));
-        	}
-
-        	inverse_alignment_col_freqs = (ARRAY_T*) retrieve_object(
-          		alignment_species,
-          		alignment_col_freqs_list
-        	);
-      	}
+          inverse_alignment_col_freqs = (ARRAY_T*) retrieve_object(
+              alignment_species,
+              alignment_col_freqs_list
+          );
+        }
       }
       /* ------------------------------------------------------------------- */
 
-      free_tree(TRUE, trimmed_tree);
+      free_tree(true, trimmed_tree);
 
       // Free list of species
       free_string_list(alignment_species);
@@ -807,47 +822,46 @@ int main(int argc, char *argv[]) {
       // to an index into ungapped reference sequence.
       int* coord_conv_table = make_alignment_to_seq_table(
           current_ref_seq_index,
-					trimmed_alignment
-        );
-      
-      if (use_BLS) {
+          trimmed_alignment);
 
-         n_scored += bls_score_sequence_in_alignment(
-           current_ref_seq_index,
-           trimmed_alignment, 
-           motif->id,
-           tree, // tree
-           window_size,
-           NULL, // background frequencies
-           models, 
-           pssm,
-           inverse_motif->id,
-	   inverse_models,
-           inverse_pssm,
-           coord_conv_table,
-           gap_support, 
-           gap_cost,
-           output_pthresh,
-	   bls_distance,
-           scanned_seq
+      if (use_BLS) {
+        n_scored += bls_score_sequence_in_alignment(
+            alph,
+            current_ref_seq_index,
+            trimmed_alignment, 
+            (scan_both_strands ? get_motif_st_id(motif) : get_motif_id(motif)),
+            tree, // tree
+            window_size,
+            NULL, // background frequencies
+            models, 
+            pssm,
+            (scan_both_strands ? get_motif_st_id(inverse_motif) : get_motif_id(motif)),
+            inverse_models,
+            inverse_pssm,
+            coord_conv_table,
+            gap_support, 
+            gap_cost,
+            output_pthresh,
+            bls_distance,
+            scanned_seq
          );
 
       } else {
-
-         n_scored += score_sequence_in_alignment(
-           current_ref_seq_index,
-           trimmed_alignment, 
-           motif->id,
-           NULL, // tree
-           window_size,
-           models, 
-           NULL, // background frequencies
-           pssm,
-           coord_conv_table,
-           gap_support, 
-           gap_cost,
-           output_pthresh,
-           scanned_seq
+        n_scored += score_sequence_in_alignment(
+            alph,
+            current_ref_seq_index,
+            trimmed_alignment, 
+            (scan_both_strands ? get_motif_st_id(motif) : get_motif_id(motif)),
+            NULL, // tree
+            window_size,
+            models, 
+            NULL, // background frequencies
+            pssm,
+            coord_conv_table,
+            gap_support, 
+            gap_cost,
+            output_pthresh,
+            scanned_seq
          );
       }
       
@@ -858,12 +872,12 @@ int main(int argc, char *argv[]) {
     // Doesn't work for the BLS scores
     set_pattern_is_complete(pattern);
     if (compute_qvalues && !use_BLS) {
-      pattern_calculate_qvalues(pattern);
+      pattern_calculate_qvalues(pattern, NULL);
     }
 
     if (models != NULL) {
       int model_index;
-      int num_models = motif->length + 1;
+      int num_models = get_motif_length(motif) + 1;
       for (model_index = 0; model_index < num_models; model_index++) {
         free_model(models[model_index]);
       }
@@ -880,7 +894,7 @@ int main(int argc, char *argv[]) {
     print_cisml_as_text(cisml);
   }
   else {
-    print_full_results(
+    if (! print_full_results(
       cisml,
       output_dirname,
       "motiph.xml",
@@ -888,25 +902,25 @@ int main(int argc, char *argv[]) {
       "motiph.txt",
       "motiph.gff",
       allow_clobber,
-      TRUE
-    );
+      true
+    )) report_err = true;
   }
 
   /**********************************************
    * Clean up.
    **********************************************/
-  for (motif_index = 0; motif_index < num_motifs; motif_index++) {
-    free_matrix(motifs[motif_index].freqs);
-  }
+  free_motifs(motifs);
   free_object_list(alignment_col_freqs_list);
   free_array(bg_freqs);
-  free_tree(TRUE, tree);
+  free_tree(true, tree);
   free_string_list(filenames);
   free_string_list(selected_motifs);
   free_cisml(cisml);
 
   //FIXME: (tlb) this is useful for checking p-value accuracy
-  fprintf(stderr, "Total number of positions scored: %d\n", n_scored);
+  if (verbosity >= NORMAL_VERBOSE) {
+    fprintf(stderr, "Total number of positions scored: %d\n", n_scored);
+  }
 
-  return(0);
+  return(report_err ? 1 : 0);
 } // main
