@@ -11,6 +11,6 @@
 # define mtype_h
 
 /* types of models */
-typedef enum {Tcm, Oops, Zoops} MTYPE;
+typedef enum {Tcm, Oops, Zoops} MOTYPE;
 
 #endif /* mtype_h */
