@@ -579,6 +579,10 @@ static int local_max(
       // FIXME: We are assuming that priors are always symmetrical here.
       int prob = pY[j]+log_not_o[j]; 		/* log (pY * Pr(site) * Pr(no overlap)) */
       if (j==next_j) n_maxima++;		/* candidate not exceeded */
+      if (n_maxima >= MAX_LOCAL_MAXIMA) {	/* limit number of local maxima */
+        n_maxima--;
+        break;
+      }
       if (j==next_j || prob>max) {		/* create/overwrite */
         max = prob;				/* new max */
         maxima[n_maxima].x = i;			/* overwrite the candidate */
