@@ -29,6 +29,9 @@ function options_reset() {
 }
 
 function fix_reset() {
+  // Make sure "Hidden Modifications" gets turned off on form reset.
+  var i, more_opts = document.getElementsByClassName("more_opts");
+  for (i=0; i<more_opts.length; i++) { toggle_class(more_opts[i], 'modified', false); }
 }
 
 function on_form_submit(evt) {
