@@ -467,7 +467,7 @@ public class MotifValidator {
     while (!(cursor.isEndElement() && cursor.getLocalName().equals("motifs"))) {
       cursor.require(START_ELEMENT, null, "motif");
       Attr aMotifWidth = new AttrRegex("width", "^(\\d+)$");
-      tagAtrs(cursor, true, feedback, new Attr("id"), new Attr("name"), new Attr("alt", true), aMotifWidth, new Attr("sites"), new Attr("ic"), new Attr("re"), new Attr("llr"), new Attr("p_value", true), new Attr("e_value"), new Attr("bayes_threshold"), new Attr("elapsed_time"), new Attr("url", true));
+      tagAtrs(cursor, true, feedback, new Attr("id"), new Attr("name"), new Attr("alt", true), aMotifWidth, new Attr("sites"), new Attr("ic"), new Attr("re"), new Attr("llr"), new Attr("p_value", true), new Attr("e_value"), new Attr("bayes_threshold"), new Attr("elapsed_time"), new Attr("url", true), new Attr("total_sites", true), new Attr("site_distr", true));
       int motifWidth;
       try {
         motifWidth = Integer.parseInt(aMotifWidth.getValue());
@@ -600,7 +600,7 @@ public class MotifValidator {
         cursor.require(START_ELEMENT, null, "letter");
         Attr aAliases = new Attr("aliases", true), aColour = new Attr("colour", true),
             aComplement = new Attr("complement", true), aEquals = new Attr("equals", true), aId = new Attr("id"),
-            aName = new Attr("name", true), aSymbol = new Attr("symbol");
+            aName = new Attr("name", true), aSymbol = new AttrRegex("symbol", "^(.)$");
         tagAtrs(cursor, true, null, aAliases, aColour, aComplement, aEquals, aId, aName, aSymbol);
         alphParser.parseSymbol(aSymbol.getValue(), aName.getValue(), aColour.getValue(), aComplement.getValue(),
             aEquals.getValue(), aAliases.getValue());
@@ -700,6 +700,7 @@ public class MotifValidator {
   }
 
   private static void parseStremeXML(MotifStats stats, XMLStreamReader cursor, FeedbackHandler feedback) throws XMLStreamException, AlphParser.AlphParseException {
+    Map<String,String> id2sym = new TreeMap<String, String>();
     cursor.require(START_ELEMENT, null, "STREME");
     tagAtrs(cursor, true, null, new AttrRegex("version", "(\\d+\\.\\d+\\.\\d+)"), new Attr("release"));
     nextTag(cursor, feedback);
@@ -734,11 +735,11 @@ public class MotifValidator {
     nextTag(cursor, feedback);
     cursor.require(END_ELEMENT, null, "test_negatives");
     nextTag(cursor, feedback);
-    cursor.require(START_ELEMENT, null, "sequence_db");
-    nextTag(cursor, feedback);
-
+    if (cursor.getLocalName().equals("sequence_db")) {		// Moved down after 5.3.0
+      cursor.require(START_ELEMENT, null, "sequence_db"); nextTag(cursor, feedback); cursor.require(END_ELEMENT, null, "sequence_db");
+      nextTag(cursor, feedback);
+    }
     Alph alph;
-    nextTag(cursor, feedback);
     if (cursor.getLocalName().equals("alphabet")) {
       AlphParser alphParser = new AlphParser();
       cursor.require(START_ELEMENT, null, "alphabet");
@@ -750,9 +751,10 @@ public class MotifValidator {
       while (cursor.getLocalName().equals("letter")) {
         cursor.require(START_ELEMENT, null, "letter");
         Attr aAliases = new Attr("aliases", true), aColour = new Attr("colour", true),
-            aComplement = new Attr("complement", true), aEquals = new Attr("equals", true), aId = new Attr("id"),
-            aName = new Attr("name", true), aSymbol = new Attr("symbol");
+	  aComplement = new Attr("complement", true), aEquals = new Attr("equals", true), aId = new Attr("id"),
+	  aName = new Attr("name", true), aSymbol = new AttrRegex("symbol", "^(.)$");
         tagAtrs(cursor, true, null, aAliases, aColour, aComplement, aEquals, aId, aName, aSymbol);
+        id2sym.put(aId.getValue(), aSymbol.getValue());
         alphParser.parseSymbol(aSymbol.getValue(), aName.getValue(), aColour.getValue(), aComplement.getValue(),
             aEquals.getValue(), aAliases.getValue());
         nextTag(cursor, feedback); cursor.require(END_ELEMENT, null, "letter");
@@ -769,9 +771,26 @@ public class MotifValidator {
       alph = AlphStd.DNA.getAlph(); // if the alphabet is not specified then assume DNA
     }
     stats.setAlphabet(alph);
-    cursor.require(START_ELEMENT, null, "background");
-    nextTag(cursor, feedback); cursor.require(END_ELEMENT, null, "background");
-    nextTag(cursor, feedback); cursor.require(START_ELEMENT, null, "stop");
+    if (cursor.getLocalName().equals("sequence_db")) {		// Moved here after 5.3.0
+      cursor.require(START_ELEMENT, null, "sequence_db");
+      nextTag(cursor, feedback);
+      nextTag(cursor, feedback);
+    }
+    if (cursor.getLocalName().equals("background")) {		// 5.3.0 
+      cursor.require(START_ELEMENT, null, "background");
+      nextTag(cursor, feedback); cursor.require(END_ELEMENT, null, "background");
+    } else {
+      cursor.require(START_ELEMENT, null, "background_frequencies");
+      tagAtrs(cursor, true, null, new Attr("source"), new Attr("order"));
+      nextTag(cursor, feedback);
+      cursor.require(START_ELEMENT, null, "alphabet_array");
+      parseAlphabetArrayMemeXML(cursor, feedback, id2sym, alph);
+      cursor.require(END_ELEMENT, null, "alphabet_array");
+      nextTag(cursor, feedback);
+      cursor.require(END_ELEMENT, null, "background_frequencies");
+    }
+    nextTag(cursor, feedback); 
+    cursor.require(START_ELEMENT, null, "stop");
     tagAtrs(cursor, true, null, new Attr("count", true), new Attr("evalue", true), new Attr("time", true));
     nextTag(cursor, feedback); cursor.require(END_ELEMENT, null, "stop");
     nextTag(cursor, feedback);
@@ -795,8 +814,20 @@ public class MotifValidator {
     nextTag(cursor, feedback); cursor.require(START_ELEMENT, null, "cand"); cursor.getElementText(); cursor.require(END_ELEMENT, null, "cand");
     nextTag(cursor, feedback); cursor.require(START_ELEMENT, null, "experimental"); cursor.getElementText(); cursor.require(END_ELEMENT, null, "experimental");
     nextTag(cursor, feedback); cursor.require(START_ELEMENT, null, "totallength"); cursor.getElementText(); cursor.require(END_ELEMENT, null, "totallength");
+    nextTag(cursor, feedback); 
+    if (cursor.getLocalName().equals("align")) {		// added in 5.4.0 
+      cursor.require(START_ELEMENT, null, "align"); 
+      cursor.getElementText(); 
+      cursor.require(END_ELEMENT, null, "align");
+    }
     nextTag(cursor, feedback); cursor.require(START_ELEMENT, null, "host"); cursor.getElementText(); cursor.require(END_ELEMENT, null, "host");
-    nextTag(cursor, feedback);
+    nextTag(cursor, feedback); 
+    if (cursor.getLocalName().equals("description")) {		// optional
+      cursor.require(START_ELEMENT, null, "description"); 
+      cursor.getElementText(); 
+      cursor.require(END_ELEMENT, null, "description");
+      nextTag(cursor, feedback);
+    }
     cursor.require(END_ELEMENT, null, "model");
     nextTag(cursor, feedback);
     cursor.require(START_ELEMENT, null, "motifs");
@@ -842,12 +873,12 @@ public class MotifValidator {
       while(!cursor.isStartElement()) cursor.next();
       if (cursor.getLocalName().equals("MEME")) {
         parseMemeXML(stats, cursor, feedback);
-      } else if (cursor.getLocalName().equals("dreme")) {
-        parseDremeXML(stats, cursor, feedback);
       } else if (cursor.getLocalName().equals("STREME")) {
         parseStremeXML(stats, cursor, feedback);
+      } else if (cursor.getLocalName().equals("dreme")) {
+        parseDremeXML(stats, cursor, feedback);
       } else {
-        throw new MotifParseException("Expected a MEME or DREME XML file.");
+        throw new MotifParseException("Expected a MEME, STREME or DREME XML file.");
       }
     } catch (XMLStreamException e) {
       Throwable nestErr = e.getNestedException();
@@ -875,6 +906,9 @@ public class MotifValidator {
     if (cursor.isProperty("stop_reason")) {
       cursor.nextStringProperty("stop_reason");
     }
+    if (cursor.isProperty("description")) {
+      cursor.nextStringProperty("description");
+    }
     cursor.nextListProperty("cmd", String.class);
     cursor.skipProperty("options"); // skip over options
     Object alphabetObj = cursor.nextProperty("alphabet");
@@ -897,21 +931,21 @@ public class MotifValidator {
       }
       stats.setAlphabet(alphParser.parseDone());
     }
-    if (cursor.isProperty("background")) { // start background
+    if (cursor.isProperty("background")) { 		// start background
       cursor.next();
       cursor.next(JSON_START_OBJECT);
       if (cursor.isProperty("source")) cursor.nextStringProperty("source");
       if (cursor.isProperty("order")) cursor.nextNumberProperty("order");
       cursor.nextListProperty("freqs", Number.class);
       cursor.next(JSON_END_OBJECT);
-      cursor.next(JSON_END_PROPERTY); // end background
+      cursor.next(JSON_END_PROPERTY); 			// end background
     }
     if (isMeme) {
-      cursor.next("sequence_db"); // start sequence_db
+      cursor.next("sequence_db"); 			// start sequence_db
       cursor.next(JSON_START_OBJECT);
       if (cursor.isProperty("source")) {		// pre 5.0
         cursor.nextStringProperty("source");
-      } else {					// post 5.0
+      } else {						// post 5.0
         cursor.nextStringProperty("primary_source");
         cursor.nextNumberProperty("primary_count");
         cursor.nextNumberProperty("primary_positions");
@@ -990,9 +1024,27 @@ public class MotifValidator {
         cursor.nextNumberProperty("test_neg_count");
         cursor.nextNumberProperty("test_log_pvalue");
         cursor.nextStringProperty("test_pvalue");
+        if (cursor.isProperty("test_log_evalue")) {	// post 5.4
+          cursor.skipProperty("test_log_evalue");
+        }
+        if (cursor.isProperty("test_evalue")) {		// post 5.4
+          cursor.skipProperty("test_evalue");
+        }
         cursor.nextNumberProperty("test_dtc");
         cursor.nextNumberProperty("test_bernoulli");
         cursor.nextNumberProperty("elapsed_time");
+	if (cursor.isProperty("total_sites")) {		// post 5.4
+	  cursor.skipProperty("total_sites");
+	}
+	if (cursor.isProperty("site_distr")) {		// post 5.4
+	  cursor.skipProperty("site_distr");
+	}
+	if (cursor.isProperty("max_sites")) {		// post 5.4
+	  cursor.skipProperty("max_sites");
+	}
+	if (cursor.isProperty("site_hist")) {		// post 5.4
+	  cursor.skipProperty("site_hist");
+	}
         cursor.nextNumberProperty("len");
         cursor.nextNumberProperty("nsites");
         cursor.nextStringProperty("evalue");
